/*
 * Copyright (c) 2008, The JUNG Authors
 * All rights reserved.
 * 
 * This software is open-source under the BSD license; see either "license.txt"
 * or https://github.com/jrtom/jung/blob/master/LICENSE for a description.
 */
package edu.uci.ics.jung.samples;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;

import javax.swing.JFrame;

import com.google.common.base.Supplier;

import edu.uci.ics.jung.algorithms.layout.FRLayout;
import edu.uci.ics.jung.graph.DirectedSparseGraph;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.UndirectedSparseGraph;
import edu.uci.ics.jung.io.PajekNetReader;
import edu.uci.ics.jung.visualization.VisualizationViewer;
import edu.uci.ics.jung.visualization.decorators.ToStringLabeller;

/**
 * A class that shows the minimal work necessary to load and visualize a graph.
 */
public class SimpleGraphDraw 
{

    @SuppressWarnings({ "rawtypes", "unchecked" })
	public static void main(String[] args) throws IOException {
        JFrame jf = new JFrame();
        Graph g = getGraph();
        VisualizationViewer vv = new VisualizationViewer(new FRLayout(g));
        vv.getRenderContext().setVertexLabelTransformer(new ToStringLabeller());
        vv.getRenderContext().setEdgeLabelTransformer(new ToStringLabeller());

        jf.getContentPane().add(vv);
        jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        jf.pack();
        jf.setVisible(true);
    }
    
    /**
     * Generates a graph: in this case, reads it from the file
     * "samples/datasetsgraph/simple.net"
     * @return A sample undirected graph
     * @throws IOException if there is an error in reading the file
     */
    @SuppressWarnings({ "rawtypes", "unchecked" })
	public static Graph getGraph() throws IOException 
    {
        PajekNetReader pnr = new PajekNetReader(new Supplier(){
			public Object get() {
				return new Object();
			}});

        Graph g = new UndirectedSparseGraph();
//        Graph g = new DirectedSparseGraph();

        URL fileUrl = SimpleGraphDraw.class.getClassLoader().getResource("datasets/weighted.net");
        URI fileUri = null;
        try {
            fileUri =  fileUrl != null ? fileUrl.toURI() : null;
        } catch (URISyntaxException e) { }
        String filename = fileUri != null ? fileUri.getPath() : null;
        pnr.load(filename, g);


        return g;
    }
}
