package pl.edu.wat.wcy.isi.szymonm.phd.graph.adjacency;

import cern.colt.matrix.impl.SparseDoubleMatrix2D;
import com.google.common.collect.BiMap;
import edu.uci.ics.jung.algorithms.operations.GraphMatrixOperations;
import edu.uci.ics.jung.algorithms.util.Indexer;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.UndirectedGraph;
import org.apache.poi.xssf.usermodel.*;
import pl.edu.wat.wcy.isi.szymonm.phd.enums.notes.NoteNamesOrdered;
import pl.edu.wat.wcy.isi.szymonm.phd.graph.structure.WeightedEdge;
import pl.edu.wat.wcy.isi.szymonm.phd.graph.structure.WeightedVertex;

import java.awt.*;
import java.io.*;
import java.util.Collection;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

public class GraphAdjacencyMatrixAssembler {

    public static final int FIRST_ROW_OFFSET = 1;
    public static final int FIRST_VERTEX_ROW_OFFSET = 45;
    public static final int FIRST_VERTEX_COLUMN_OFFSET = 2;
    public static final int SECOND_VERTEX_ROW_OFFSET = 46;
    public static final int SECOND_VERTEX_COLUMN_OFFSET = 1;
    public static final int FIRST_MATRIX_FIRST_ROW_OFFSET = 140;
    public static final int SECOND_MATRIX_FIRST_ROW_OFFSET = 165;
    public static final int FIRST_PART_CELL_OFFSET = 2;
    public static final int SECOND_PART_CELL_OFFSET = 25;
    public static final String WORKSHEET_MAX_FIT_NAME = "A_strukt";
    public static final String WORKSHEET_GOAL_NAME = "A_strukt_celowe";
    public static final String WORKSHEET_THRESHOLD_NAME = "A_strukt_progowe";
    private static final int PARAMS_COLNUM = 22;
    private static final int PARAM_STRUCTURAL_WEIGHT_ROWNUM = 23;
    private static final int PARAM_VERTEX_WEIGHT_ROWNUM = 24;
    private static final int PARAM_EDGE_WEIGHT_ROWNUM = 25;
    private static final double STRUCTURAL_WEIGHT_VALUE = 0.34;
    private static final double VERTEX_WEIGHT_VALUE = 0.33;
    private static final double EDGE_WEIGHT_VALUE = 0.33;

    public void createAdjacencyMatrixUsingOperations(UndirectedGraph<String, String> graph) {
        // TODO szymonm dodac sztucznie brakujace wierzcholki i zapewnic jednoznacznosc ich odwzorowania!
        System.out.println(GraphMatrixOperations.graphToSparseMatrix(graph).toString());
    }

        /**
         * Metoda sortuje wierzchołki zgodnie z nazewnictwem nut (C, Cis, D, Dis..., H, Pauza)
         */
    public void createAdjacencyMatrix(Graph<String, String> graph) {
        Collection<String> vertices = graph.getVertices();
        List<String> orderedVertices = orderNotes(vertices);
        int i = 0;
        int lastNotNullRow = 0;

        if (vertices.size() == 0) {
            System.out.println("macierz zerowa");
            return;
        }
        if (vertices.size() == 1) {
            System.out.println("pojedynczy wierzchołek");
            return;
        }

        for (String node: orderedVertices) {
            System.out.println(node);
            int nodeIndex = getNoteIndexFromNoteName(node);
            if (nodeIndex != i) {
                for (int k = lastNotNullRow; k <= nodeIndex - lastNotNullRow; k++) {
                    System.out.println("Row " + (k + i) + " - empty");
                    //printEmptyLine(k);
//                    i++;
                }
            }

            Collection<String> neighborVertices = graph.getNeighbors(node);
            if ((long) neighborVertices.size() == 0) {
                System.out.println(i + " - empty");
            }

            List<String> orderedNeighbors = orderNotes(neighborVertices);
            System.out.print("wiersz " + i + " :");
            for (String noteName: orderedNeighbors) {
                int noteIndex = getNoteIndexFromNoteName(noteName);
                System.out.print(" " + noteIndex);
            }
            System.out.println();
            i++;
            lastNotNullRow = nodeIndex;
        }
    }

    public void createAdjMatrix(Graph<String, String> graph) {
        Collection<String> vertices = graph.getVertices();
        List<String> orderedVertices = orderNotes(vertices);
        int i = 0;
        int lastNotNullRow = 0;

        if (vertices.size() == 0) {
            System.out.println("macierz zerowa");
            return;
        }
        if (vertices.size() == 1) {
            System.out.println("pojedynczy wierzchołek");
            return;
        }

        SparseDoubleMatrix2D matrix = new SparseDoubleMatrix2D(vertices.size(),
                vertices.size());

        BiMap<String,Integer> indexer = Indexer.<String>create(orderedVertices);
        for(String v : orderedVertices)
        {
            System.out.println(v);
            Collection<String> outEdges = graph.getOutEdges(v);
            List<String> orderedOutEdges = orderEdges(outEdges);
            for (String e : orderedOutEdges)
            {
                String w = graph.getOpposite(v,e);
                int j = indexer.get(w);
                System.out.println(e);
                System.out.println("Setting i=" + i + " and j=" + j + " to: " +(matrix.getQuick(i,j) + 1));
                matrix.set(i, j, matrix.getQuick(i,j) + 1);
            }
            i++;
        }
        for(String v : orderedVertices) {
            System.out.print(v + addSeparatorIfNeeded(v));
        }
        System.out.println(matrix.toString());
    }

    public void createAdjMatrixWithExcelFile(Graph<String, String> graph, File excelFile, int partIndex) {
        Collection<String> vertices = graph.getVertices();
        List<String> orderedVertices = orderNotes(vertices);
        int i = 0;
        int lastNotNullRow = 0;

        if (vertices.size() == 0) {
            System.out.println("macierz zerowa");
            return;
        }
        if (vertices.size() == 1) {
            System.out.println("pojedynczy wierzchołek");
            return;
        }

        SparseDoubleMatrix2D matrix = new SparseDoubleMatrix2D(vertices.size(),
                vertices.size());

        BiMap<String,Integer> indexer = Indexer.<String>create(orderedVertices);
        for(String v : orderedVertices)
        {
            System.out.println(v);
            Collection<String> outEdges = graph.getOutEdges(v);
            List<String> orderedOutEdges = orderEdges(outEdges);
            for (String e : orderedOutEdges)
            {
                String w = graph.getOpposite(v,e);
                int j = indexer.get(w);
                System.out.println(e);
                System.out.println("Setting i=" + i + " and j=" + j + " to: " +(matrix.getQuick(i,j) + 1));
                matrix.set(i, j, matrix.getQuick(i,j) + 1);
            }
            i++;
        }
        for(String v : orderedVertices) {
            System.out.print(v + addSeparatorIfNeeded(v));
        }
        System.out.println(matrix.toString());


        replaceInFileAndOpen(matrix, excelFile, partIndex, WORKSHEET_MAX_FIT_NAME, true);
    }

    public void createAdjMatrixForNetworkWithExcelFile(Graph<String, WeightedEdge> graph, File excelFile, int partIndex) {
        Collection<String> vertices = graph.getVertices();
        List<String> orderedVertices = orderNotes(vertices);
        int i = 0;
        int lastNotNullRow = 0;

        if (vertices.size() == 0) {
            System.out.println("macierz zerowa");
            return;
        }
        if (vertices.size() == 1) {
            System.out.println("pojedynczy wierzchołek");
            return;
        }

        SparseDoubleMatrix2D adjMatrix = new SparseDoubleMatrix2D(vertices.size(),
                vertices.size());
        SparseDoubleMatrix2D edgeMatrix = new SparseDoubleMatrix2D(vertices.size(),
                vertices.size());

        BiMap<String,Integer> indexer = Indexer.<String>create(orderedVertices);
        for(String v : orderedVertices)
        {
            System.out.println(v);
            Collection<WeightedEdge> outEdges = graph.getOutEdges(v);
            List<WeightedEdge> orderedOutEdges = orderWeightedEdges(outEdges);
            for (WeightedEdge e : orderedOutEdges)
            {
                String w = graph.getOpposite(v,e);
                int j = indexer.get(w);
                System.out.println(e);
                int weight = e.getWeight();
                System.out.println("Setting i=" + i + " and j=" + j + " to: " +(edgeMatrix.getQuick(i,j) + weight));
                adjMatrix.set(i, j, adjMatrix.getQuick(i,j) + 1);
                edgeMatrix.set(i, j, edgeMatrix.getQuick(i,j) + weight);
            }
            i++;
        }
        for(String v : orderedVertices) {
            System.out.print(v + addSeparatorIfNeeded(v));
        }
        System.out.println(adjMatrix.toString());
        System.out.println(edgeMatrix.toString());


        replaceEdgesInFileAndOpen(edgeMatrix, excelFile, partIndex, WORKSHEET_MAX_FIT_NAME);
        replaceInFileAndOpen(adjMatrix, excelFile, partIndex, WORKSHEET_MAX_FIT_NAME, true);
    }

    public void createAdjMatrixForWeightedNetworkWithExcelFile(Graph<WeightedVertex, WeightedEdge> graph, File excelFile, int partIndex) {
        Collection<WeightedVertex> vertices = graph.getVertices();
        List<WeightedVertex> orderedVertices = orderWeightedNotes(vertices);
        int i = 0;
        int lastNotNullRow = 0;

        if (vertices.size() == 0) {
            System.out.println("macierz zerowa");
            return;
        }
        if (vertices.size() == 1) {
            System.out.println("pojedynczy wierzchołek");
            return;
        }

        SparseDoubleMatrix2D adjMatrix = new SparseDoubleMatrix2D(vertices.size(),
                vertices.size());
        SparseDoubleMatrix2D edgeMatrix = new SparseDoubleMatrix2D(vertices.size(),
                vertices.size());

        BiMap<WeightedVertex,Integer> indexer = Indexer.<WeightedVertex>create(orderedVertices);
        for(WeightedVertex v : orderedVertices)
        {
            System.out.println(v);
            Collection<WeightedEdge> outEdges = graph.getOutEdges(v);
            List<WeightedEdge> orderedOutEdges = orderWeightedEdges(outEdges);
            for (WeightedEdge e : orderedOutEdges)
            {
                WeightedVertex w = graph.getOpposite(v,e);
                int j = indexer.get(w);
                System.out.println(e);
                int weight = e.getWeight();
                System.out.println("Setting i=" + i + " and j=" + j + " to: " +(edgeMatrix.getQuick(i,j) + weight));
                adjMatrix.set(i, j, adjMatrix.getQuick(i,j) + 1);
                edgeMatrix.set(i, j, edgeMatrix.getQuick(i,j) + weight);
            }
            i++;
        }
        for(WeightedVertex v : orderedVertices) {
            System.out.print(v + addSeparatorIfNeeded(v.getId()));
        }
        System.out.println(adjMatrix.toString());
        System.out.println(edgeMatrix.toString());


        replaceWeightedEdgesInFileAndOpen(edgeMatrix, graph, excelFile, partIndex, WORKSHEET_MAX_FIT_NAME);
        replaceInFileAndOpen(adjMatrix, excelFile, partIndex, WORKSHEET_MAX_FIT_NAME, false);
        replaceWeightedEdgesInFileAndOpen(edgeMatrix, graph, excelFile, partIndex, WORKSHEET_GOAL_NAME);
        replaceInFileAndOpen(adjMatrix, excelFile, partIndex, WORKSHEET_GOAL_NAME, false);
        replaceWeightedEdgesInFileAndOpen(edgeMatrix, graph, excelFile, partIndex, WORKSHEET_THRESHOLD_NAME);
        replaceInFileAndOpen(adjMatrix, excelFile, partIndex, WORKSHEET_THRESHOLD_NAME, true);
    }

    private void replaceInFileAndOpen(SparseDoubleMatrix2D matrix, File excelFile, int partIndex, String worksheetName,
                                      boolean shouldOpenFile) {
        XSSFWorkbook workbook = null;
        try (FileInputStream fip = new FileInputStream(excelFile)) {
            workbook = new XSSFWorkbook(fip);
        } catch (IOException e) {
            e.printStackTrace();
        }
        workbook.setForceFormulaRecalculation(true);
        XSSFSheet workbookSheet = workbook.getSheet(worksheetName);
        for (int i = 0; i < 20; i++) {
            XSSFRow row = workbookSheet.getRow(i + FIRST_ROW_OFFSET);

            for(int j = 0; j < 20; j++) {
                int offset = partIndex == 0 ? FIRST_PART_CELL_OFFSET : SECOND_PART_CELL_OFFSET;
                XSSFCell cell = row.getCell(j + offset);
                if (i < 13 && j < 13) {
                    cell.setCellValue(matrix.getQuick(i, j));
                } else {
                    cell.setCellValue(0);
                }
            }
        }
        XSSFRow row = workbookSheet.getRow(2);
        XSSFCell cell = row.getCell(22);
        cell.setCellValue(13);
        XSSFRow row1 = workbookSheet.getRow(2);
        XSSFCell cell1 = row1.getCell(45);
        cell1.setCellValue(13);
        // refreshing other, dependent cells calculated using these values above:
        //XSSFFormulaEvaluator.evaluateAllFormulaCells(workbook);

        OutputStream out = null;
        try {
            out = new FileOutputStream(excelFile);
            workbook.write(out);
            out.close();
            if (shouldOpenFile && partIndex != 0) {
                Desktop.getDesktop().open(new File(excelFile.getAbsolutePath()));
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void replaceEdgesInFileAndOpen(SparseDoubleMatrix2D matrix, File excelFile, int partIndex, String worksheetName) {
        XSSFWorkbook workbook = null;
        try (FileInputStream fip = new FileInputStream(excelFile)) {
            workbook = new XSSFWorkbook(fip);
        } catch (IOException e) {
            e.printStackTrace();
        }
        workbook.setForceFormulaRecalculation(true);
        XSSFSheet workbookSheet = workbook.getSheet(worksheetName);
        for (int i = 0; i < 20; i++) {
            int rowOffset = partIndex == 0 ? FIRST_MATRIX_FIRST_ROW_OFFSET : SECOND_MATRIX_FIRST_ROW_OFFSET;
            XSSFRow row = workbookSheet.getRow(i + rowOffset);

            for(int j = 0; j < 20; j++) {
                XSSFCell cell = row.getCell(j + FIRST_PART_CELL_OFFSET);
                if (i < 13 && j < 13) {
                    cell.setCellValue(matrix.getQuick(i, j));
                } else {
                    cell.setCellValue(0);
                }
            }
        }
        XSSFRow row = workbookSheet.getRow(23);
        XSSFCell cell = row.getCell(22);
        cell.setCellValue(0);

        XSSFRow row1 = workbookSheet.getRow(25);
        XSSFCell cell1 = row1.getCell(22);
        cell1.setCellValue(0);
    }

    private void setEdgeMatrixValues(SparseDoubleMatrix2D matrix, int partIndex, XSSFSheet workbookSheet) {
        for (int i = 0; i < 20; i++) {
            int rowOffset = partIndex == 0 ? FIRST_MATRIX_FIRST_ROW_OFFSET : SECOND_MATRIX_FIRST_ROW_OFFSET;
            XSSFRow row = workbookSheet.getRow(i + rowOffset);

            for(int j = 0; j < 20; j++) {
                XSSFCell cell = row.getCell(j + FIRST_PART_CELL_OFFSET);
                if (i < 13 && j < 13) {
                    cell.setCellValue(matrix.getQuick(i, j));
                } else {
                    cell.setCellValue(0);
                }
            }
        }
    }

    private void replaceWeightedEdgesInFileAndOpen(SparseDoubleMatrix2D matrix, Graph<WeightedVertex, WeightedEdge> graph,
                                                   File excelFile, int partIndex, String worksheetName) {
        XSSFWorkbook workbook = null;
        try (FileInputStream fip = new FileInputStream(excelFile)) {
            workbook = new XSSFWorkbook(fip);
        } catch (IOException e) {
            e.printStackTrace();
        }
        workbook.setForceFormulaRecalculation(true);
        XSSFSheet workbookSheet = workbook.getSheet(worksheetName);

        setEdgeMatrixValues(matrix, partIndex, workbookSheet);

        setVertexMatrixValues(graph, partIndex, workbookSheet);

        setMethodComparisonWeights(workbookSheet);

        // refreshing other, dependent cells calculated using these values above:
        //XSSFFormulaEvaluator.evaluateAllFormulaCells(workbook);

        OutputStream out = null;
        try {
            out = new FileOutputStream(excelFile);
            workbook.write(out);
            out.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void setMethodComparisonWeights(XSSFSheet workbookSheet) {
        XSSFRow row = workbookSheet.getRow(PARAM_STRUCTURAL_WEIGHT_ROWNUM);
        XSSFCell cell = row.getCell(PARAMS_COLNUM);
        cell.setCellValue(STRUCTURAL_WEIGHT_VALUE);

        XSSFRow row1 = workbookSheet.getRow(PARAM_VERTEX_WEIGHT_ROWNUM);
        XSSFCell cell1 = row1.getCell(PARAMS_COLNUM);
        cell1.setCellValue(VERTEX_WEIGHT_VALUE);

        XSSFRow row2 = workbookSheet.getRow(PARAM_EDGE_WEIGHT_ROWNUM);
        XSSFCell cell2 = row2.getCell(PARAMS_COLNUM);
        cell2.setCellValue(EDGE_WEIGHT_VALUE);
    }

    private void setVertexMatrixValues(Graph<WeightedVertex, WeightedEdge> graph, int partIndex, XSSFSheet workbookSheet) {
        final NoteNamesOrdered[] noteNamesOrdereds = NoteNamesOrdered.values();
        if (partIndex == 0) {

            for (int i = 0; i < 20; i++) {

                XSSFRow row = workbookSheet.getRow(FIRST_VERTEX_ROW_OFFSET);
                XSSFCell cell = row.getCell(i + FIRST_VERTEX_COLUMN_OFFSET);

                if (i < 13) {
                   final String currentNoteName = i < 12 ? noteNamesOrdereds[i].name() : "P";
                   WeightedVertex foundVertex = graph.getVertices().stream()
                            .filter(vertex -> vertex.getId().equalsIgnoreCase(currentNoteName))
                            .findFirst()
                            .orElseThrow(() -> new RuntimeException("Vertex with name " + currentNoteName + " was not found "));
                    cell.setCellValue(foundVertex.getWeight());
                } else {
                    cell.setCellValue(0);
                }
            }
        } else {
            for (int i = 0; i < 20; i++) {
                XSSFRow row = workbookSheet.getRow(SECOND_VERTEX_ROW_OFFSET + i);
                XSSFCell cell = row.getCell(SECOND_VERTEX_COLUMN_OFFSET);

                if (i < 13) {
                    final String currentNoteName = i < 12 ? noteNamesOrdereds[i].name() : "P";
                    WeightedVertex foundVertex = graph.getVertices().stream()
                            .filter(vertex -> vertex.getId().equalsIgnoreCase(currentNoteName))
                            .findFirst()
                            .orElseThrow(() -> new RuntimeException("Vertex with name " + currentNoteName + " was not found "));
                    cell.setCellValue(foundVertex.getWeight());
                } else {
                    cell.setCellValue(0);
                }
            }
        }
    }

    private String addSeparatorIfNeeded(String noteName) {
        return noteName.equalsIgnoreCase("E")
                || noteName.equalsIgnoreCase("A")
                || noteName.equalsIgnoreCase("B")
                || noteName.equalsIgnoreCase("H")
                || noteName.equalsIgnoreCase("P") ? " " : "";
    }

    Comparator<String> noteNameComparator () {
        return (o1, o2) -> {
            if (o1.equalsIgnoreCase(o2)) {
                throw new IllegalArgumentException("W prostej reprezentacji nie mogą wystąpić dwa identyczne wierzchołki!");
            }
            if (o1.equalsIgnoreCase("P")) {
                return 1;
            }
            if (o2.equalsIgnoreCase("P")) {
                return -1;
            }

            NoteNamesOrdered firstNoteName = NoteNamesOrdered.valueOf(o1);
            NoteNamesOrdered secondNoteName = NoteNamesOrdered.valueOf(o2);
            int firstNoteOrdinal = firstNoteName.ordinal();
            int secondNoteOrdinal = secondNoteName.ordinal();

            return firstNoteOrdinal - secondNoteOrdinal;
        };
    }

    Comparator<WeightedVertex> noteVertexNameComparator () {
        return (o1, o2) -> {
            if (o1.getId().equalsIgnoreCase(o2.getId())) {
                throw new IllegalArgumentException("W prostej reprezentacji nie mogą wystąpić dwa identyczne wierzchołki!");
            }
            if (o1.getId().equalsIgnoreCase("P")) {
                return 1;
            }
            if (o2.getId().equalsIgnoreCase("P")) {
                return -1;
            }

            NoteNamesOrdered firstNoteName = NoteNamesOrdered.valueOf(o1.getId());
            NoteNamesOrdered secondNoteName = NoteNamesOrdered.valueOf(o2.getId());
            int firstNoteOrdinal = firstNoteName.ordinal();
            int secondNoteOrdinal = secondNoteName.ordinal();

            return firstNoteOrdinal - secondNoteOrdinal;
        };
    }

    Comparator<String> edgeComparator () {
        return (o1, o2) -> {
            if (o1.equalsIgnoreCase(o2)) {
                throw new IllegalArgumentException("W prostej reprezentacji nie mogą wystąpić dwa identyczne wierzchołki!");
            }

            String firstNeighbor = o1.substring(o1.indexOf("-") + 1);
            String secondNeighbor = o2.substring(o2.indexOf("-") + 1);

            if (firstNeighbor.equalsIgnoreCase("P")) {
                return 1;
            }
            if (secondNeighbor.equalsIgnoreCase("P")) {
                return -1;
            }
            NoteNamesOrdered firstNoteName = NoteNamesOrdered.valueOf(firstNeighbor);
            NoteNamesOrdered secondNoteName = NoteNamesOrdered.valueOf(secondNeighbor);
            int firstNoteOrdinal = firstNoteName.ordinal();
            int secondNoteOrdinal = secondNoteName.ordinal();

            return firstNoteOrdinal - secondNoteOrdinal;
        };
    }

    Comparator<WeightedEdge> weightedEdgeComparator () {
        return (o1, o2) -> {
            if (o1.getId().equalsIgnoreCase(o2.getId())) {
                throw new IllegalArgumentException("W prostej reprezentacji nie mogą wystąpić dwa identyczne wierzchołki!");
            }

            String firstNeighbor = o1.getId().substring(o1.getId().indexOf("-") + 1);
            String secondNeighbor = o2.getId().substring(o2.getId().indexOf("-") + 1);

            if (firstNeighbor.equalsIgnoreCase("P")) {
                return 1;
            }
            if (secondNeighbor.equalsIgnoreCase("P")) {
                return -1;
            }
            NoteNamesOrdered firstNoteName = NoteNamesOrdered.valueOf(firstNeighbor);
            NoteNamesOrdered secondNoteName = NoteNamesOrdered.valueOf(secondNeighbor);
            int firstNoteOrdinal = firstNoteName.ordinal();
            int secondNoteOrdinal = secondNoteName.ordinal();

            return firstNoteOrdinal - secondNoteOrdinal;
        };
    }

    int getNoteIndexFromNoteName(String noteName) {
        if (noteName.equalsIgnoreCase("P")) {
            return 12;
        }
        NoteNamesOrdered noteNameEnum = NoteNamesOrdered.valueOf(noteName);
        return noteNameEnum.ordinal();
    }

    List<String> orderNotes(Collection<String> vertices) {
        return vertices.stream()
                .sorted(noteNameComparator())
                .collect(Collectors.toList());
    }

    List<WeightedVertex> orderWeightedNotes(Collection<WeightedVertex> vertices) {
        return vertices.stream()
                .sorted(noteVertexNameComparator())
                .collect(Collectors.toList());
    }

    List<String> orderEdges(Collection<String> edges) {
        return edges.stream()
                .sorted(edgeComparator())
                .collect(Collectors.toList());
    }

    List<WeightedEdge> orderWeightedEdges(Collection<WeightedEdge> edges) {
        return edges.stream()
                .sorted(weightedEdgeComparator())
                .collect(Collectors.toList());
    }

}
