package pl.edu.wat.wcy.isi.szymonm.phd.graph.comparison.note;

import pl.edu.wat.wcy.isi.szymonm.phd.enums.notes.RhythmValues;
import pl.edu.wat.wcy.isi.szymonm.phd.musicxml.Note;

public class NoteNameConverter {

    /**
     * Returns note mapped to a single character (preserving differences between octaves, durations and note names)
     */
    public String convertNoteToString(Note note) {
        int noteNumber = this.getNumberForNoteName(note);
        int octaveMultiplier = this.getNumberForNoteOctave(note);
        int durationsMultiplier = RhythmValues.values().length;
        int durationOffset = getNumberForNoteDuration(note);

        int characterIndex = noteNumber * octaveMultiplier + durationsMultiplier * durationsMultiplier * durationOffset;
        char character = (char) characterIndex;
        return String.valueOf(character);
    }

    public int getNumberForNoteName(Note note) {
        return note.getNoteIndex();
    }

    private int getNumberForNoteDuration(Note note) {
        return note.getRhythmValuesValue().ordinal();
    }

    private int getNumberForNoteOctave(Note note) {
        return this.getNumberForOctave(note.getNoteFullName());
    }

    protected int getNumberForOctave(String s) {
        // TODO change colors to octave numbers
        if(s.contains("P"))
            return 0;

        if (s.contains("₂"))
            return 1;
        if (s.contains("₁"))
            return 2;
        if (s.contains("¹"))
            return 5;
        if (s.contains("²"))
            return 6;
        if (s.contains("³"))
            return 7;
        if (s.contains("⁴"))
            return 8;
        if (s.contains("⁵"))
            return 9;
        if (s.contains("⁶"))
            return 10;

        // if none of above works, it is great or small octave; if first letter is uppercase then it's great octave
        if (s.substring(0,1).equals(s.substring(0,1).toUpperCase()))
            return 3;
        else
            return 4;

    }


}
