package pl.edu.wat.wcy.isi.szymonm.phd.graph.structure;

import com.google.common.base.Functions;
import edu.uci.ics.jung.algorithms.layout.CircleLayout;
import edu.uci.ics.jung.algorithms.metrics.Metrics;
import edu.uci.ics.jung.algorithms.metrics.Statistics;
import edu.uci.ics.jung.algorithms.scoring.BetweennessCentrality;
import edu.uci.ics.jung.algorithms.shortestpath.DistanceStatistics;
import edu.uci.ics.jung.graph.DirectedGraph;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.GraphDecorator;
import edu.uci.ics.jung.visualization.GraphZoomScrollPane;
import edu.uci.ics.jung.visualization.VisualizationViewer;
import edu.uci.ics.jung.visualization.control.*;
import edu.uci.ics.jung.visualization.decorators.ToStringLabeller;
import edu.uci.ics.jung.visualization.renderers.BasicEdgeLabelRenderer;
import edu.uci.ics.jung.visualization.renderers.BasicVertexLabelRenderer;
import edu.uci.ics.jung.visualization.renderers.GradientVertexRenderer;
import edu.uci.ics.jung.visualization.renderers.Renderer;
import pl.edu.wat.wcy.isi.szymonm.phd.enums.notes.RhythmValues;
import pl.edu.wat.wcy.isi.szymonm.phd.musicxml.Note;
import pl.edu.wat.wcy.isi.szymonm.phd.musicxml.Score;

import javax.swing.*;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.Unmarshaller;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

public abstract class BaseGraphNotesCreation {

    VisualizationViewer<String,String> vv;
    VisualizationViewer<String,WeightedEdge> vwe;
    VisualizationViewer<WeightedVertex,WeightedEdge> vwvwe;

    public abstract void displayGraph(List<Note> notes, String filename) throws Exception;
    public abstract Graph<String, String> addNotesToGraph(List<Note> notes);
    public abstract Graph<String, WeightedEdge> addNotesToNetwork(List<Note> notes);
    public abstract Graph<WeightedVertex, WeightedEdge> addNotesToWeightedNetwork(List<Note> notes);
    public abstract int getMaxEdgeCount();
    public abstract int getMaxVertexCount();

    protected void setEnvironment(Graph<String,String> graph, String filename) throws IOException {

        // create a simple graph for the demo
        vv = new VisualizationViewer<String, String>(new CircleLayout<>(graph));

        vv.addGraphMouseListener((GraphMouseListener<String>) new DirectedGraphStandardNotesCreation.TestGraphMouseListener<String>());
        vv.getRenderer().setVertexRenderer(
                new GradientVertexRenderer<String,String>(
                        Color.white, Color.red,
                        Color.white, Color.blue,
                        vv.getPickedVertexState(),
                        false));

        // add my listeners for ToolTips
        vv.setVertexToolTipTransformer(new ToStringLabeller(){
            @Override
            public String apply(Object o) {
                return super.apply(o);
            }
        });

        vv.getRenderContext().setVertexLabelTransformer(Functions.identity());
        // vv.getRenderContext().setEdgeLabelTransformer(Function.identity());
/*
        vv.getRenderContext().setEdgeLabelTransformer(new Function<Number, String>()  {
            public String apply(Number o) {
                String s = gmlr.getEdgeDescriptions().get(o);
                if(s==null)
                    return "1";
                return s;
            }
        });
*/

        vv.getRenderContext().setEdgeFontTransformer(Functions.constant(new Font("Helvetica", Font.PLAIN, 18)));
        vv.getRenderContext().setVertexFontTransformer(Functions.constant(new Font("Helvetica", Font.PLAIN, 18)));

        // multigraph only!
//        vv.getRenderContext().setEdgeShapeTransformer(EdgeShape.quadCurve(graph));

        //vv.getRenderContext().setVertexLabelTransformer(new ToStringLabeller());
        vv.getRenderer().getVertexLabelRenderer().setPositioner(new BasicVertexLabelRenderer.InsidePositioner());
        vv.getRenderer().getVertexLabelRenderer().setPosition(edu.uci.ics.jung.visualization.renderers.Renderer.VertexLabel.Position.AUTO);

        // create a frome to hold the graph
        final JFrame frame = new JFrame();
        Container content = frame.getContentPane();
        final GraphZoomScrollPane panel = new GraphZoomScrollPane(vv);
        content.add(panel);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        final AbstractModalGraphMouse graphMouse = new DefaultModalGraphMouse<Number,Number>();
        vv.setGraphMouse(graphMouse);
        vv.addKeyListener(graphMouse.getModeKeyListener());

        JMenuBar menubar = new JMenuBar();
        menubar.add(graphMouse.getModeMenu());
        panel.setCorner(menubar);


        vv.addKeyListener(graphMouse.getModeKeyListener());
        vv.setToolTipText("<html><center>Type 'p' for Pick mode<p>Type 't' for Transform mode");

        final ScalingControl scaler = new CrossoverScalingControl();

        JButton plus = new JButton("+");
        plus.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                scaler.scale(vv, 1.1f, vv.getCenter());
            }
        });
        JButton minus = new JButton("-");
        minus.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                scaler.scale(vv, 1/1.1f, vv.getCenter());
            }
        });

        JPanel controls = new JPanel();
        controls.add(plus);
        controls.add(minus);
        content.add(controls, BorderLayout.SOUTH);

        frame.pack();
        frame.setTitle(filename);
        frame.setVisible(true);

        String[] filePath = filename.split("/");
        printStatistics(graph, filePath);
    }

    private void printStatistics(Graph<String, String> graph, String[] filePath) {
        System.out.println("Graf " + filePath[filePath.length-1]);
        System.out.println("Liczba gałęzi: " + graph.getEdgeCount());
        System.out.println("Liczba wierzchołków: " + graph.getVertexCount());
        System.out.println("Wsp. klasteryzacji (śr, min, max): " + Statistics.mean(Metrics.clusteringCoefficients(graph).values())
                + ", " + Statistics.min(Metrics.clusteringCoefficients(graph).values())
                + ", " + Statistics.max(Metrics.clusteringCoefficients(graph).values()));

        ArrayList<Integer> degrees = new ArrayList<>();
        GraphDecorator<String, String> g = new GraphDecorator<>(graph);

        for(String v: g.getVertices()) degrees.add(g.degree(v));
        System.out.println("Sredni stopień: " + Statistics.mean(degrees));

        degrees = new ArrayList<>();
        for(String v: g.getVertices()) degrees.add(g.inDegree(v));
        System.out.println("Sredni stopień wchodzący: " + Statistics.mean(degrees));

        degrees = new ArrayList<>();
        for(String v: g.getVertices()) degrees.add(g.outDegree(v));
        System.out.println("Sredni stopień wychodzący: " + Statistics.mean(degrees));

        System.out.println("Srednica: " + DistanceStatistics.diameter(graph));

        int n = graph.getVertexCount();
        int potentialEdgeCount;
        if(graph instanceof DirectedGraph)
            potentialEdgeCount = n*(n-1) + n; //pełny w klasie <1, 0, 1>
        else
            potentialEdgeCount = (n*(n-1))/2 + n; //pełny w klasie <0, 1, 1>

        System.out.println("Gęstość: " + (double) graph.getEdgeCount()/potentialEdgeCount);

        ArrayList<Double> betweenness = new ArrayList<>();
        for(String v: g.getVertices()) betweenness.add(new BetweennessCentrality(graph).getVertexScore(v));
        System.out.println("Sredni betweenness wierzchołków: " + Statistics.mean(betweenness));

        betweenness = new ArrayList<>();
        for(String e: g.getEdges()) betweenness.add(new BetweennessCentrality(graph).getEdgeScore(e));
        System.out.println("Sredni betweenness gałęzi: " + Statistics.mean(betweenness));
    }

    protected void setNetworkEnvironment(Graph<String,WeightedEdge> graph, String filename) throws IOException {

        // create a simple graph for the demo
        vwe =  new VisualizationViewer<String,WeightedEdge>(new CircleLayout<>(graph));
        vwe.addGraphMouseListener((GraphMouseListener<String>) new DirectedGraphStandardNotesCreation.TestGraphMouseListener<String>());
        vwe.getRenderer().setVertexRenderer(
                new GradientVertexRenderer<String,WeightedEdge>(
                        Color.white, Color.red,
                        Color.white, Color.blue,
                        vwe.getPickedVertexState(),
                        false)
        );

        // add my listeners for ToolTips
        vwe.setVertexToolTipTransformer(new ToStringLabeller(){
            @Override
            public String apply(Object o) {
                return super.apply(o);
            }
        });

        vwe.getRenderContext().setVertexLabelTransformer(Functions.identity());
        // vv.getRenderContext().setEdgeLabelTransformer(Function.identity());

        vwe.getRenderContext().setEdgeLabelTransformer(o -> String.valueOf(o.getWeight()));

        vwe.getRenderContext().setEdgeFontTransformer(Functions.constant(new Font("Helvetica", Font.PLAIN, 18)));
        vwe.getRenderContext().setVertexFontTransformer(Functions.constant(new Font("Helvetica", Font.PLAIN, 18)));

        // multigraph only!
//        vv.getRenderContext().setEdgeShapeTransformer(EdgeShape.quadCurve(graph));

        //vv.getRenderContext().setVertexLabelTransformer(new ToStringLabeller());
        vwe.getRenderer().getVertexLabelRenderer().setPositioner(new BasicVertexLabelRenderer.InsidePositioner());
        vwe.getRenderer().getVertexLabelRenderer().setPosition(edu.uci.ics.jung.visualization.renderers.Renderer.VertexLabel.Position.AUTO);

        // create a frome to hold the graph
        final JFrame frame = new JFrame();
        Container content = frame.getContentPane();
        final GraphZoomScrollPane panel = new GraphZoomScrollPane(vwe);
        content.add(panel);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        final AbstractModalGraphMouse graphMouse = new DefaultModalGraphMouse<Number,Number>();
        vwe.setGraphMouse(graphMouse);
        vwe.addKeyListener(graphMouse.getModeKeyListener());

        JMenuBar menubar = new JMenuBar();
        menubar.add(graphMouse.getModeMenu());
        panel.setCorner(menubar);


        vwe.addKeyListener(graphMouse.getModeKeyListener());
        vwe.setToolTipText("<html><center>Type 'p' for Pick mode<p>Type 't' for Transform mode");

        final ScalingControl scaler = new CrossoverScalingControl();

        JButton plus = new JButton("+");
        plus.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                scaler.scale(vwe, 1.1f, vwe.getCenter());
            }
        });
        JButton minus = new JButton("-");
        minus.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                scaler.scale(vwe, 1/1.1f, vwe.getCenter());
            }
        });

        JPanel controls = new JPanel();
        controls.add(plus);
        controls.add(minus);
        content.add(controls, BorderLayout.SOUTH);

        frame.pack();
        frame.setTitle(filename);
        frame.setVisible(true);

        String[] filePath = filename.split("/");
        printNetworkStatistics(graph, filePath);
    }

    private void printNetworkStatistics(Graph<String, ?> graph, String[] filePath) {
        System.out.println("Graf " + filePath[filePath.length-1]);
        System.out.println("Liczba gałęzi: " + graph.getEdgeCount());
        System.out.println("Liczba wierzchołków: " + graph.getVertexCount());
        System.out.println("Wsp. klasteryzacji (śr, min, max): " + Statistics.mean(Metrics.clusteringCoefficients(graph).values())
                + ", " + Statistics.min(Metrics.clusteringCoefficients(graph).values())
                + ", " + Statistics.max(Metrics.clusteringCoefficients(graph).values()));

        ArrayList<Integer> degrees = new ArrayList<>();
        GraphDecorator<String, ?> g = new GraphDecorator<>(graph);

        for(String v: g.getVertices()) degrees.add(g.degree(v));
        System.out.println("Sredni stopień: " + Statistics.mean(degrees));

        degrees = new ArrayList<>();
        for(String v: g.getVertices()) degrees.add(g.inDegree(v));
        System.out.println("Sredni stopień wchodzący: " + Statistics.mean(degrees));

        degrees = new ArrayList<>();
        for(String v: g.getVertices()) degrees.add(g.outDegree(v));
        System.out.println("Sredni stopień wychodzący: " + Statistics.mean(degrees));

        System.out.println("Srednica: " + DistanceStatistics.diameter(graph));

        int n = graph.getVertexCount();
        int potentialEdgeCount;
        if(graph instanceof DirectedGraph)
            potentialEdgeCount = n*(n-1) + n; //pełny w klasie <1, 0, 1>
        else
            potentialEdgeCount = (n*(n-1))/2 + n; //pełny w klasie <0, 1, 1>

        System.out.println("Gęstość: " + (double) graph.getEdgeCount()/potentialEdgeCount);

        ArrayList<Double> betweenness = new ArrayList<>();
        for(String v: g.getVertices()) betweenness.add(new BetweennessCentrality(graph).getVertexScore(v));
        System.out.println("Sredni betweenness wierzchołków: " + Statistics.mean(betweenness));

        betweenness = new ArrayList<>();
        for(Object e: g.getEdges()) betweenness.add(new BetweennessCentrality(graph).getEdgeScore(e));
        System.out.println("Sredni betweenness gałęzi: " + Statistics.mean(betweenness));
    }


    protected void setWeightedNetworkEnvironment(Graph<WeightedVertex,WeightedEdge> graph, String filename) throws IOException {

        // create a simple graph for the demo
        vwvwe =  new VisualizationViewer<WeightedVertex,WeightedEdge>(new CircleLayout<>(graph));
        vwvwe.addGraphMouseListener(new DirectedGraphStandardNotesCreation.TestGraphMouseListener<WeightedVertex>());
        vwvwe.getRenderer().setVertexRenderer(
                new GradientVertexRenderer<WeightedVertex,WeightedEdge>(
                        Color.white, Color.red,
                        Color.white, Color.blue,
                        vwvwe.getPickedVertexState(),
                        false)
        );

        // add my listeners for ToolTips
        vwvwe.setVertexToolTipTransformer(new ToStringLabeller(){
            @Override
            public String apply(Object o) {
                return super.apply(o);
            }
        });

        vwvwe.getRenderContext().setVertexLabelTransformer(v -> v.getId() + " (" + v.getWeight() + ")");
        // vv.getRenderContext().setEdgeLabelTransformer(Function.identity());

        vwvwe.getRenderContext().setEdgeLabelTransformer(o -> String.valueOf(o.getWeight()));

        vwvwe.getRenderContext().setEdgeFontTransformer(Functions.constant(new Font("Helvetica", Font.PLAIN, 18)));
        vwvwe.getRenderContext().setVertexFontTransformer(Functions.constant(new Font("Helvetica", Font.PLAIN, 18)));

        // multigraph only!
//        vv.getRenderContext().setEdgeShapeTransformer(EdgeShape.quadCurve(graph));

        //vv.getRenderContext().setVertexLabelTransformer(new ToStringLabeller());
        vwvwe.getRenderer().getVertexLabelRenderer().setPositioner(new BasicVertexLabelRenderer.OutsidePositioner());
        vwvwe.getRenderer().getVertexLabelRenderer().setPosition(Renderer.VertexLabel.Position.S);

        vwvwe.getRenderer().setEdgeLabelRenderer(new BasicEdgeLabelRenderer<>());

        // create a frome to hold the graph
        final JFrame frame = new JFrame();
        Container content = frame.getContentPane();
        final GraphZoomScrollPane panel = new GraphZoomScrollPane(vwvwe);
        content.add(panel);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        final AbstractModalGraphMouse graphMouse = new DefaultModalGraphMouse<Number,Number>();
        vwvwe.setGraphMouse(graphMouse);
        vwvwe.addKeyListener(graphMouse.getModeKeyListener());

        JMenuBar menubar = new JMenuBar();
        menubar.add(graphMouse.getModeMenu());
        panel.setCorner(menubar);


        vwvwe.addKeyListener(graphMouse.getModeKeyListener());
        vwvwe.setToolTipText("<html><center>Type 'p' for Pick mode<p>Type 't' for Transform mode");

        final ScalingControl scaler = new CrossoverScalingControl();

        JButton plus = new JButton("+");
        plus.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                scaler.scale(vwe, 1.1f, vwvwe.getCenter());
            }
        });
        JButton minus = new JButton("-");
        minus.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                scaler.scale(vwe, 1/1.1f, vwvwe.getCenter());
            }
        });

        JPanel controls = new JPanel();
        controls.add(plus);
        controls.add(minus);
        content.add(controls, BorderLayout.SOUTH);

        frame.pack();
        frame.setTitle(filename);
        frame.setVisible(true);
    }

    /**
     * A nested class to demo the GraphMouseListener finding the
     * right vertices after zoom/pan
     */
    static class TestGraphMouseListener<V> implements GraphMouseListener<V> {

        public void graphClicked(V v, MouseEvent me) {
            System.err.println("Vertex "+v+" was clicked at ("+me.getX()+","+me.getY()+")");
        }
        public void graphPressed(V v, MouseEvent me) {
            System.err.println("Vertex "+v+" was pressed at ("+me.getX()+","+me.getY()+")");
        }
        public void graphReleased(V v, MouseEvent me) {
            System.err.println("Vertex "+v+" was released at ("+me.getX()+","+me.getY()+")");
        }
    }


    public List<Note> getNotesFromFile(String file) throws Exception {
        //given
        URL fileUrl = this.getClass().getClassLoader().getResource(file);
        URI fileUri = null;
        try {
            fileUri =  fileUrl != null ? fileUrl.toURI() : null;
        } catch (URISyntaxException e) { }
        String filename = fileUri != null ? fileUri.getPath() : file;

        JAXBContext context = JAXBContext.newInstance(Score.class);
        Unmarshaller um = context.createUnmarshaller();
        Score score = (Score) um.unmarshal(new File(filename));

        List<Note> notes = score.getPartList().get(0).getPhraseList().get(0).getNoteList();
        refreshFrequencyForAllNotes(notes);

        for(int i=0; i < notes.size(); i++){
            if(notes.get(i).getRhythmValuesValue().equals(RhythmValues.UNKNOWN)){
                int prevNote = (i > 0) ? i - 1 : 0;
                notes.get(i).setRhythmValue(notes.get(prevNote).getRhythmValue());
            }
            if(notes.get(i).getRhythmValuesValue().equals(RhythmValues.UNKNOWN)){
                int nextNote = (i < notes.size()-2) ? i + 1 : i;
                notes.get(i).setRhythmValue(notes.get(nextNote).getRhythmValue());
            }
            if(notes.get(i).getRhythmValuesValue().equals(RhythmValues.UNKNOWN)){
                notes.get(i).setRhythmValue(1);
            }
        }

        return notes;
    }

    public List<Double> getGraphCharacteristics(Graph<String, ?> graph) {
        List<Double> characteristics = new ArrayList<>();

        int edgeCount = graph.getEdgeCount();
        System.out.println("Liczba gałęzi: " + edgeCount);
        characteristics.add((double)edgeCount);


        int vertexCount = graph.getVertexCount();
        System.out.println("Liczba wierzchołków: " + vertexCount);
        characteristics.add((double)vertexCount);


        double meanClusteringCoef = Statistics.mean(Metrics.clusteringCoefficients(graph).values());
        double minClusteringCoef = Statistics.min(Metrics.clusteringCoefficients(graph).values());
        double maxClusteringCoef = Statistics.max(Metrics.clusteringCoefficients(graph).values());
        System.out.println("Wsp. klasteryzacji (śr, min, max): " + meanClusteringCoef
                + ", " + minClusteringCoef
                + ", " + maxClusteringCoef);
        characteristics.add(meanClusteringCoef);
        characteristics.add(minClusteringCoef);
        characteristics.add(maxClusteringCoef);


        ArrayList<Integer> degrees = new ArrayList<>();
        GraphDecorator<String, ?> g = new GraphDecorator<>(graph);

        for(String v: g.getVertices())
            degrees.add(g.degree(v));
        double meanDegree = Statistics.mean(degrees);
        System.out.println("Sredni stopień: " + meanDegree);
        characteristics.add(meanDegree);

        degrees = new ArrayList<>();
        for(String v: g.getVertices())
            degrees.add(g.inDegree(v));
        System.out.println("Sredni stopień wchodzący: " + Statistics.mean(degrees));
        double meanInDegree = Statistics.mean(degrees);
        characteristics.add(meanInDegree);

        degrees = new ArrayList<>();
        for(String v: g.getVertices()) degrees.add(g.outDegree(v));
        System.out.println("Sredni stopień wychodzący: " + Statistics.mean(degrees));
        double meanOutDegree = Statistics.mean(degrees);
        characteristics.add(meanOutDegree);

        double diameter =  DistanceStatistics.diameter(graph);
        System.out.println("Srednica: " + diameter);
        characteristics.add(diameter);

        int n = graph.getVertexCount();
        int potentialEdgeCount;
        if(graph instanceof DirectedGraph)
            potentialEdgeCount = n*(n-1) + n; //pełny w klasie <1, 0, 1>
        else
            potentialEdgeCount = (n*(n-1))/2 + n; //pełny w klasie <0, 1, 1>

        System.out.println("Gęstość: " + (double)graph.getEdgeCount()/potentialEdgeCount);

        ArrayList<Double> betweenness = new ArrayList<>();
        for(String v: g.getVertices()) betweenness.add(new BetweennessCentrality(graph).getVertexScore(v));
        System.out.println("Sredni betweenness wierzchołków: " + Statistics.mean(betweenness));
        double meanBetweenness = Statistics.mean(betweenness);
        characteristics.add(meanBetweenness);

        betweenness = new ArrayList<>();
        for(Object e: g.getEdges()) betweenness.add(new BetweennessCentrality(graph).getEdgeScore(e));
        double meanEdgeBetweenness = Statistics.mean(betweenness);
        System.out.println("Sredni betweenness gałęzi: " + meanEdgeBetweenness);
        characteristics.add(meanEdgeBetweenness);

        return characteristics;
    }

    public List<Double> getGraphCharacteristicsNormalized(Graph<String, ?> graph){
        List<Double> characteristicsNormalized = new ArrayList<>();

        int edgeCount = graph.getEdgeCount();
        int possibleEdgeCount = this.getMaxEdgeCount();
        System.out.println("Liczba gałęzi: " + edgeCount);
        double edgeCountNormalized = (double)edgeCount / possibleEdgeCount;
        characteristicsNormalized.add(edgeCountNormalized);


        int vertexCount = graph.getVertexCount();
        int possibleVertexCount = getMaxVertexCount();
        System.out.println("Liczba wierzchołków: " + vertexCount);
        double vertexCountNormalized = (double)vertexCount / possibleVertexCount;
        characteristicsNormalized.add(vertexCountNormalized);


        double meanClusteringCoef = Statistics.mean(Metrics.clusteringCoefficients(graph).values());
        double minClusteringCoef = Statistics.min(Metrics.clusteringCoefficients(graph).values());
        double maxClusteringCoef = Statistics.max(Metrics.clusteringCoefficients(graph).values());
        System.out.println("Wsp. klasteryzacji (śr, min, max): " + meanClusteringCoef
                + ", " + minClusteringCoef
                + ", " + maxClusteringCoef);
        characteristicsNormalized.add(meanClusteringCoef);
        characteristicsNormalized.add(minClusteringCoef);
        characteristicsNormalized.add(maxClusteringCoef);


        ArrayList<Integer> degrees = new ArrayList<>();
        GraphDecorator<String, ?> g = new GraphDecorator<>(graph);

        for(String v: g.getVertices())
            degrees.add(g.degree(v));
        double meanDegree = Statistics.mean(degrees);
        System.out.println("Sredni stopień: " + meanDegree);
        double meanDegreeNormalized = meanDegree/getMaxVertexCount();
        characteristicsNormalized.add(meanDegreeNormalized);

        degrees = new ArrayList<>();
        for(String v: g.getVertices())
            degrees.add(g.inDegree(v));
        System.out.println("Sredni stopień wchodzący: " + Statistics.mean(degrees));
        double meanInDegree = Statistics.mean(degrees);
        double meanInDegreeNormalized = meanInDegree/getMaxVertexCount();
        characteristicsNormalized.add(meanInDegreeNormalized);

        degrees = new ArrayList<>();
        for(String v: g.getVertices()) degrees.add(g.outDegree(v));
        System.out.println("Sredni stopień wychodzący: " + Statistics.mean(degrees));
        double meanOutDegree = Statistics.mean(degrees);
        double meanOutDegreeNormalized = meanOutDegree/getMaxVertexCount();
        characteristicsNormalized.add(meanOutDegreeNormalized);

        double diameter =  DistanceStatistics.diameter(graph);
        System.out.println("Srednica: " + diameter);
        double diameterNormalized = diameter/getMaxVertexCount();
        characteristicsNormalized.add(diameterNormalized);

        int n = graph.getVertexCount();
        int potentialEdgeCount;
        if(graph instanceof DirectedGraph)
            potentialEdgeCount = n*(n-1) + n; //pełny w klasie <1, 0, 1>
        else
            potentialEdgeCount = (n*(n-1))/2 + n; //pełny w klasie <0, 1, 1>

        System.out.println("Gęstość: " + (double)graph.getEdgeCount()/potentialEdgeCount);

        ArrayList<Double> betweenness = new ArrayList<>();
        for(String v: g.getVertices()) betweenness.add(new BetweennessCentrality(graph).getVertexScore(v));
        System.out.println("Sredni betweenness wierzchołków: " + Statistics.mean(betweenness));
        double meanBetweennessNormalized = Statistics.mean(betweenness)/(getMaxVertexCount()*getMaxVertexCount());
        characteristicsNormalized.add(meanBetweennessNormalized);

        betweenness = new ArrayList<>();
        for(Object e: g.getEdges()) betweenness.add(new BetweennessCentrality(graph).getEdgeScore(e));
        double meanEdgeBetweenness = Statistics.mean(betweenness);
        System.out.println("Sredni betweenness gałęzi: " + meanEdgeBetweenness);
        double meanEdgeBetweennessNormalized = meanEdgeBetweenness/(getMaxVertexCount()*getMaxVertexCount());
        characteristicsNormalized.add(meanEdgeBetweennessNormalized);

        return characteristicsNormalized;
    }

    private void refreshFrequencyForAllNotes(List<Note> noteList) {
        // adjusts frequency using pitch
        noteList.forEach(note -> note.setPitch(note.getPitch()));
    }

}
