package pl.edu.wat.wcy.isi.szymonm.phd.graph.structure;

import edu.uci.ics.jung.graph.DirectedSparseGraph;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.DirectedGraph;
import edu.uci.ics.jung.visualization.VisualizationViewer;
import pl.edu.wat.wcy.isi.szymonm.phd.enums.notes.NoteNames;
import pl.edu.wat.wcy.isi.szymonm.phd.musicxml.Note;

import java.util.List;

public class DirectedGraphSimpleNotesCreation extends BaseGraphNotesCreation {

    VisualizationViewer<String,String> vv;

    public void displayGraph(List<Note> notes, String filename) throws Exception {
        DirectedGraph<String,String> graph = addNotesToGraph(notes);
        setEnvironment(graph, filename);
    }

    @Override
    public Graph<String, WeightedEdge> addNotesToNetwork(List<Note> notes) {
        throw new UnsupportedOperationException("Not available for graph structure");
    }

    @Override
    public Graph<WeightedVertex, WeightedEdge> addNotesToWeightedNetwork(List<Note> notes) {
        throw new UnsupportedOperationException("Not available for graph structure");
    }

    public DirectedGraph<String,String> addNotesToGraph(List<Note> notes, boolean withNonExistent) {
        DirectedGraph<String, String> DirectedGraphWithNotesFromPart = addNotesToGraph(notes);
        if (withNonExistent) {
            addNonExistentNotes(DirectedGraphWithNotesFromPart);
        }

        return DirectedGraphWithNotesFromPart;
    }

    private void addNonExistentNotes(DirectedGraph<String, String> DirectedGraphWithNotesFromPart) {
        for (NoteNames note: NoteNames.values()) {
            if (!DirectedGraphWithNotesFromPart.containsVertex(note.name())) {
                DirectedGraphWithNotesFromPart.addVertex(note.name());
            }
        }

        if (!DirectedGraphWithNotesFromPart.containsVertex("P")) {
            DirectedGraphWithNotesFromPart.addVertex("P");
        }
    }

    public DirectedGraph<String,String> addNotesToGraph(List<Note> notes) {
        DirectedGraph<String,String> graph = new DirectedSparseGraph<>();

        if (notes.isEmpty()) {
            return graph;
        }

        for(int i = 0; i < notes.size(); i++){
            Note note = notes.get(i);
            String noteName = note.getNoteShortName();
            if(i == 0) {
                graph.addVertex(noteName);
            } else {
                if(note.getPitch() == 0)
                    continue;

                Note prevNote = notes.get(i-1);
                String prevNoteName = prevNote.getNoteShortName();

                if(!graph.containsVertex(note.getNoteShortName()))
                    graph.addVertex(note.getNoteShortName());

                if(graph.findEdge(prevNoteName, noteName) == null)
                    graph.addEdge(prevNoteName + "-" + noteName, prevNoteName, noteName);

            }
        }

        String firstNoteName = notes.get(0).getNoteShortName();
        String lastNoteName = notes.get(notes.size()-1).getNoteShortName();

        if(graph.findEdge(lastNoteName, firstNoteName) == null)
            graph.addEdge(lastNoteName + "-" + firstNoteName, lastNoteName, firstNoteName);

        return graph;
    }

    @Override
    public int getMaxEdgeCount() {
        return 13*13/2;
    }

    @Override
    public int getMaxVertexCount() {
        return 13;
    }
}
