package pl.edu.wat.wcy.isi.szymonm.phd.graph.structure;

import edu.uci.ics.jung.graph.DirectedGraph;
import edu.uci.ics.jung.graph.DirectedSparseGraph;
import edu.uci.ics.jung.graph.Graph;
import pl.edu.wat.wcy.isi.szymonm.phd.musicxml.Note;

import java.util.List;

public class DirectedGraphStandardNotesCreation extends BaseGraphNotesCreation {


    @Override
    public void displayGraph(List<Note> notes, String filename) throws Exception {
        Graph<String,String> graph = addNotesToGraph(notes);
        setEnvironment(graph, filename);
    }

    @Override
    public Graph<String, WeightedEdge> addNotesToNetwork(List<Note> notes) {
        throw new UnsupportedOperationException("Not available for graph structure");
    }

    @Override
    public Graph<WeightedVertex, WeightedEdge> addNotesToWeightedNetwork(List<Note> notes) {
        throw new UnsupportedOperationException("Not available for graph structure");
    }

    @Override
    public Graph<String,String> addNotesToGraph(List<Note> notes) {
        DirectedGraph<String,String> graph = new DirectedSparseGraph<>();

        if (notes.isEmpty()) {
            return graph;
        }

        for(int i = 0; i < notes.size(); i++){
            Note note = notes.get(i);
            String noteName = note.getNoteStandardName();
            if(i == 0) {
                graph.addVertex(noteName);
            } else {
                if(note.getPitch() == 0)
                    continue;

                Note prevNote = notes.get(i-1);
                String prevNoteName = prevNote.getNoteStandardName();

                if(!graph.containsVertex(note.getNoteStandardName()))
                    graph.addVertex(note.getNoteStandardName());

                if(graph.findEdge(prevNoteName, noteName) == null)
                    graph.addEdge(prevNoteName + "-" + noteName, prevNoteName, noteName);

            }
        }

        String firstNoteName = notes.get(0).getNoteStandardName();
        String lastNoteName = notes.get(notes.size()-1).getNoteStandardName();

        if(graph.findEdge(lastNoteName, firstNoteName) == null)
            graph.addEdge(lastNoteName + "-" + firstNoteName, lastNoteName, firstNoteName);

        return graph;
    }

    @Override
    public int getMaxEdgeCount() {
        return 89*90;
    }

    @Override
    public int getMaxVertexCount() {
        return 89;
    }
}
