package pl.edu.wat.wcy.isi.szymonm.phd.graph.structure;

import edu.uci.ics.jung.graph.DirectedGraph;
import edu.uci.ics.jung.graph.DirectedSparseGraph;
import edu.uci.ics.jung.graph.Graph;
import pl.edu.wat.wcy.isi.szymonm.phd.musicxml.Note;

import java.util.List;

public class DirectedNetworkFullNotesCreation extends BaseGraphNotesCreation {

    public void displayGraph(List<Note> notes, String filename) throws Exception {
        final DirectedGraph<String,WeightedEdge> graph = addNotesToNetwork(notes);
        setNetworkEnvironment(graph, filename);
    }

    @Override
    public Graph<String, String> addNotesToGraph(List<Note> notes) {
        throw new UnsupportedOperationException("Not available for network structure");
    }

    @Override
    public Graph<WeightedVertex, WeightedEdge> addNotesToWeightedNetwork(List<Note> notes) {
        throw new UnsupportedOperationException("Not available for structure with simple vertices");
    }

    public DirectedGraph<String,WeightedEdge> addNotesToNetwork(List<Note> notes) {
        DirectedGraph<String,WeightedEdge> graph = new DirectedSparseGraph<>();

        if (notes.isEmpty()) {
            return graph;
        }

        for(int i = 0; i < notes.size(); i++){
            Note note = notes.get(i);
            String noteName = note.getNoteFullName();
            if(i == 0) {
                graph.addVertex(noteName);
            } else {
                if(note.getPitch() == 0)
                    continue;

                Note prevNote = notes.get(i-1);
                String prevNoteName = prevNote.getNoteFullName();

                if(!graph.containsVertex(note.getNoteFullName()))
                    graph.addVertex(note.getNoteFullName());

                WeightedEdge edgeBetweenPrevAndCurrent = graph.findEdge(prevNoteName, noteName);
                if(edgeBetweenPrevAndCurrent == null)
                    graph.addEdge(new WeightedEdge(prevNoteName + "-" + noteName, 1), prevNoteName, noteName);
                else {
                    int weight = edgeBetweenPrevAndCurrent.getWeight();
                    graph.removeEdge(edgeBetweenPrevAndCurrent);
                    graph.addEdge(new WeightedEdge(prevNote + "-" + noteName, weight+1), prevNoteName, noteName);
                }
            }
        }

        String firstNoteName = notes.get(0).getNoteFullName();
        String lastNoteName = notes.get(notes.size()-1).getNoteFullName();

        if(graph.findEdge(lastNoteName, firstNoteName) == null)
            graph.addEdge(new WeightedEdge(lastNoteName + "-" + firstNoteName, 1), lastNoteName, firstNoteName);

        return graph;
    }

    @Override
    public int getMaxEdgeCount() {
        return 1068*1069;
    }

    @Override
    public int getMaxVertexCount() {
        return 1068;
    }
}
