package pl.edu.wat.wcy.isi.szymonm.phd.graph.structure;

import edu.uci.ics.jung.graph.DirectedGraph;
import edu.uci.ics.jung.graph.DirectedSparseGraph;
import edu.uci.ics.jung.graph.Graph;
import pl.edu.wat.wcy.isi.szymonm.phd.enums.notes.NoteNames;
import pl.edu.wat.wcy.isi.szymonm.phd.musicxml.Note;

import java.util.List;

public class DirectedNetworkShortNotesCreation extends BaseGraphNotesCreation {


    @Override
    public void displayGraph(List<Note> notes, String filename) throws Exception {
        Graph<String,WeightedEdge> graph = addNotesToNetwork(notes);
        setNetworkEnvironment(graph, filename);
    }

    @Override
    public Graph<String, String> addNotesToGraph(List<Note> notes) {
        throw new UnsupportedOperationException("Not available for network structure");
    }

    @Override
    public Graph<WeightedVertex, WeightedEdge> addNotesToWeightedNetwork(List<Note> notes) {
        throw new UnsupportedOperationException("Not available for structure with simple vertices");
    }


    public DirectedGraph<String,WeightedEdge> addNotesToNetwork(List<Note> notes, boolean withNonExistent) {
        DirectedGraph<String, WeightedEdge> directedGraphWithNotesFromPart = addNotesToNetwork(notes);
        if(withNonExistent) {
            addNonExistentNotes(directedGraphWithNotesFromPart);
        }

        return directedGraphWithNotesFromPart;
    }

    private void addNonExistentNotes(DirectedGraph<String, WeightedEdge> DirectedGraphWithNotesFromPart) {
        for (NoteNames note: NoteNames.values()) {
            if (!DirectedGraphWithNotesFromPart.containsVertex(note.name())) {
                DirectedGraphWithNotesFromPart.addVertex(note.name());
            }
        }

        if (!DirectedGraphWithNotesFromPart.containsVertex("P")) {
            DirectedGraphWithNotesFromPart.addVertex("P");
        }
    }

    @Override
    public DirectedGraph<String,WeightedEdge> addNotesToNetwork(List<Note> notes) {
        DirectedGraph<String,WeightedEdge> graph = new DirectedSparseGraph<>();

        if (notes.isEmpty()) {
            return graph;
        }

        for(int i = 0; i < notes.size(); i++){
            Note note = notes.get(i);
            String noteName = note.getNoteShortName();
            if(i == 0) {
                graph.addVertex(noteName);
            } else {
                if(note.getPitch() == 0)
                    continue;

                Note prevNote = notes.get(i-1);
                String prevNoteName = prevNote.getNoteShortName();

                if(!graph.containsVertex(note.getNoteShortName()))
                    graph.addVertex(note.getNoteShortName());

                WeightedEdge edgeBetweenPrevAndCurrent = graph.findEdge(prevNoteName, noteName);
                if(edgeBetweenPrevAndCurrent == null)
                    graph.addEdge(new WeightedEdge(prevNoteName + "-" + noteName, 1), prevNoteName, noteName);
                else {
                    int weight = edgeBetweenPrevAndCurrent.getWeight();
                    graph.removeEdge(edgeBetweenPrevAndCurrent);
                    graph.addEdge(new WeightedEdge(prevNoteName + "-" + noteName, weight+1), prevNoteName, noteName);
                }
            }
        }

        String firstNoteName = notes.get(0).getNoteShortName();
        String lastNoteName = notes.get(notes.size()-1).getNoteShortName();

        if(graph.findEdge(lastNoteName, firstNoteName) == null)
            graph.addEdge(new WeightedEdge(lastNoteName + "-" + firstNoteName, 1), firstNoteName, lastNoteName);

        return graph;
    }

    @Override
    public int getMaxEdgeCount() {
        return 89*90;
    }

    @Override
    public int getMaxVertexCount() {
        return 89;
    }
}
