package pl.edu.wat.wcy.isi.szymonm.phd.graph.structure;

import edu.uci.ics.jung.graph.DirectedGraph;
import edu.uci.ics.jung.graph.DirectedSparseGraph;
import edu.uci.ics.jung.graph.Graph;
import pl.edu.wat.wcy.isi.szymonm.phd.enums.notes.NoteNames;
import pl.edu.wat.wcy.isi.szymonm.phd.musicxml.Note;

import java.util.List;

public class DirectedNetworkWithWeightedVerticesShortNotesCreation extends BaseGraphNotesCreation {


    @Override
    public void displayGraph(List<Note> notes, String filename) throws Exception {
        Graph<WeightedVertex,WeightedEdge> graph = addAllNotesToWeightedNetwork(notes);
        setWeightedNetworkEnvironment(graph, filename);
    }

    @Override
    public Graph<String, String> addNotesToGraph(List<Note> notes) {
        throw new UnsupportedOperationException("Not available for network structure");
    }


    @Override
    public DirectedGraph<String,WeightedEdge> addNotesToNetwork(List<Note> notes) {
        throw new UnsupportedOperationException("Not available for structure with weighted vertices");
    }

    @Override
    public DirectedGraph<WeightedVertex,WeightedEdge> addNotesToWeightedNetwork(List<Note> notes) {
        return this.addNotesToWeightedNetwork(notes, true);
    }

    public DirectedGraph<WeightedVertex,WeightedEdge> addNotesToWeightedNetwork(List<Note> notes, boolean withNonExistent) {
        DirectedGraph<WeightedVertex, WeightedEdge> directedGraphWithNotesFromPart = addAllNotesToWeightedNetwork(notes);
        if(withNonExistent) {
            addNonExistentNotes(directedGraphWithNotesFromPart);
        }

        return directedGraphWithNotesFromPart;
    }

    private void addNonExistentNotes(DirectedGraph<WeightedVertex, WeightedEdge> DirectedGraphWithNotesFromPart) {
        for (NoteNames note: NoteNames.values()) {
            WeightedVertex vertex = new WeightedVertex(note.name(), 0);
            if (!DirectedGraphWithNotesFromPart.containsVertex(vertex)) {
                DirectedGraphWithNotesFromPart.addVertex(vertex);
            }
        }

        WeightedVertex pauseVertex = new WeightedVertex("P", 0);

        if (!DirectedGraphWithNotesFromPart.containsVertex(pauseVertex)) {
            DirectedGraphWithNotesFromPart.addVertex(pauseVertex);
        }
    }

    public DirectedGraph<WeightedVertex,WeightedEdge> addAllNotesToWeightedNetwork(List<Note> notes) {
        DirectedGraph<WeightedVertex,WeightedEdge> graph = new DirectedSparseGraph<>();

        if (notes.isEmpty()) {
            return graph;
        }

        for(int i = 0; i < notes.size(); i++){
            Note note = notes.get(i);
            String noteName = note.getNoteShortName();
            if(i == 0) {
                WeightedVertex vertex = new WeightedVertex(noteName, 1);
                graph.addVertex(vertex);
            } else {
                if(note.getPitch() == 0)
                    continue;

                Note prevNote = notes.get(i-1);
                String prevNoteName = prevNote.getNoteShortName();

                WeightedVertex searchedVertex = new WeightedVertex(note.getNoteShortName(), 1);
                WeightedVertex previousVertex = new WeightedVertex(prevNoteName, 1);

                if(!graph.containsVertex(searchedVertex)) {
                    graph.addVertex(searchedVertex);
                } else {
                    final String searchedName = searchedVertex.getId();
                    graph.getVertices().stream()
                            .filter(vertex -> vertex.getId().equalsIgnoreCase(searchedName))
                            .forEach(vertex -> vertex.weight++);
                }

                WeightedEdge edgeBetweenPrevAndCurrent = graph.findEdge(previousVertex, searchedVertex);
                if(edgeBetweenPrevAndCurrent == null)
                    graph.addEdge(new WeightedEdge(prevNoteName + "-" + noteName, 1), previousVertex, searchedVertex);
                else {
                    int weight = edgeBetweenPrevAndCurrent.getWeight();
                    graph.removeEdge(edgeBetweenPrevAndCurrent);
                    graph.addEdge(new WeightedEdge(prevNoteName + "-" + noteName, weight+1), previousVertex, searchedVertex);
                }
            }
        }

        String firstNoteName = notes.get(0).getNoteShortName();
        String lastNoteName = notes.get(notes.size()-1).getNoteShortName();
        WeightedVertex firtVertex = new WeightedVertex(firstNoteName, 1);
        WeightedVertex lastVertex = new WeightedVertex(lastNoteName, 1);

//        if(graph.findEdge(lastVertex, firtVertex) == null)
//            graph.addEdge(new WeightedEdge(lastNoteName + "-" + firstNoteName, 1), firtVertex, lastVertex);

        return graph;
    }

    @Override
    public int getMaxEdgeCount() {
        return 89*90;
    }

    @Override
    public int getMaxVertexCount() {
        return 89;
    }
}
