package pl.edu.wat.wcy.isi.szymonm.phd.graph.walk;

import edu.uci.ics.jung.graph.Graph;
import pl.edu.wat.wcy.isi.szymonm.phd.enums.graphs.NoteRepresentation;
import pl.edu.wat.wcy.isi.szymonm.phd.enums.notes.NoteNames;
import pl.edu.wat.wcy.isi.szymonm.phd.enums.notes.NoteNamesOrdered;
import pl.edu.wat.wcy.isi.szymonm.phd.enums.notes.OctaveNames;
import pl.edu.wat.wcy.isi.szymonm.phd.enums.notes.RhythmValues;
import pl.edu.wat.wcy.isi.szymonm.phd.musicxml.Note;
import pl.edu.wat.wcy.isi.szymonm.phd.musicxml.Part;
import pl.edu.wat.wcy.isi.szymonm.phd.musicxml.Phrase;
import pl.edu.wat.wcy.isi.szymonm.phd.musicxml.Score;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;
import java.io.File;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.*;

public abstract class BaseGraphWalkMethod<V extends String, E> {

    NoteRepresentation nodeType;

    public BaseGraphWalkMethod(NoteRepresentation nodeType) {
        this.nodeType = nodeType;
    }

    protected abstract V pickStartNode(Graph<V, E> graph);

    protected abstract V pickNextNode(Graph<V, E> graph, V currentNode);

    public List<Note> generateArrangement(Graph<V, E> graph, int arrangementNoteCount) {
        List<Note> arrangement = new ArrayList<>();
        V current = pickStartNode(graph);
        V next;

        for(int i = 0; i < arrangementNoteCount; i++){
            Note note = createNoteFromVertex(graph, current);
            arrangement.add(note);

            current = pickNextNode(graph, current);
        }

        return arrangement;
    }

    public void addNotesToFile(List<Note> notes, String originalFilename, int arrangementNumber) throws Exception {
//        int beginIndex = originalFilename.lastIndexOf('\\');
//        String fileNameWithoutPath = originalFilename;
//        if (beginIndex > 0) {
//            fileNameWithoutPath = originalFilename.substring(beginIndex + 1);
//        }
        URL fileUrl = this.getClass().getClassLoader().getResource(originalFilename);
        URI fileUri = null;
        try {
            fileUri =  fileUrl != null ? fileUrl.toURI() : null;
        } catch (URISyntaxException e) { }

        String filename = fileUri != null ? fileUri.getPath() : null;
        //filename = "C:\\Users\\szymo\\Dropbox\\Magisterka\\Test.xml";

        Calendar calendar = Calendar.getInstance();
        String dateStamp = calendar.get(Calendar.DAY_OF_MONTH) + "-" + calendar.get(Calendar.MONTH) + "-" +
                 calendar.get(Calendar.YEAR) + "_" + calendar.get(Calendar.HOUR_OF_DAY) + "-" + calendar.get(Calendar.MINUTE);
        String arrangementFilename = filename.replaceAll("\\.xml", "_arr_" + arrangementNumber + " - " + dateStamp + ".xml");

        Phrase phrase = new Phrase();
        phrase.setNoteList(notes);
        phrase.setStartTime(1.0);
        phrase.setTempo(-1.0);

        Part part = new Part();
        ArrayList<Phrase> partPhrases = new ArrayList<>();
        partPhrases.add(phrase);
        part.setPhraseList(partPhrases);

        Score score = new Score();
        ArrayList<Part> scoreParts = new ArrayList<>();
        scoreParts.add(part);
        score.setPartList(scoreParts);

        JAXBContext context = JAXBContext.newInstance(Score.class);
        Marshaller marshaller = context.createMarshaller();
        marshaller.setProperty(Marshaller.JAXB_FRAGMENT, Boolean.TRUE);
        File arrangementFile = new File(arrangementFilename);
        arrangementFile.createNewFile();
        marshaller.marshal(score, arrangementFile);

    }

    protected Note createNoteFromVertex(Graph<V, E> graph, V currentNode) {
        Note note = new Note();

        switch(this.nodeType){
            case Simplified: {
                String noteName = (currentNode.contains("is")) ? currentNode.substring(0,3) : currentNode.substring(0,1);

                int drawnRhythm = new Random().nextInt(12);
                int drawnOctave = new Random().nextInt(8);

                RhythmValues rhythmValues = RhythmValues.values()[drawnRhythm];
                String noteRhythmValue = note.getRhythmString(rhythmValues);

                NoteNamesOrdered noteNames;
                int noteValue;
                int pitch;
                if(!noteName.equals("P")){
                    noteNames = NoteNamesOrdered.valueOf(noteName);
                    noteValue = noteNames.ordinal();
                    pitch = noteValue + 12 * drawnOctave;
                } else {
                    pitch = -1;
                }

                double duration = note.getDurationFromRhythmString(noteRhythmValue);

                note.setRhythmValue(duration);
                if(pitch > 0)
                    note.setPitch(pitch);
                else
                    note.setPitch(Integer.MIN_VALUE);
                note.setDuration(duration);
                break;
            }
            case Standard: {
                String noteName = (currentNode.contains("is")) ? currentNode.substring(0,3) : currentNode.substring(0,1);
                String noteOctave;
                if("P".equals(noteName))
                    noteOctave = "";
                else
                    noteOctave = (currentNode.contains("is")) ? currentNode.substring(3,4) : currentNode.substring(1,2);

                int drawnRhythm = new Random().nextInt(12);

                RhythmValues rhythmValues = RhythmValues.values()[drawnRhythm];
                String noteRhythmValue = note.getRhythmString(rhythmValues);
                OctaveNames octaveName = note.getOctaveFromStringSymbol(noteName, noteOctave);
                int octave = octaveName.ordinal();

                NoteNamesOrdered noteNames;
                int noteValue;
                int pitch;
                if(!noteName.equals("P")){
                    noteNames = NoteNamesOrdered.valueOf(noteName);
                    noteValue = noteNames.ordinal();
                    pitch = noteValue + 12 * octave;
                } else {
                    pitch = -1;
                }

                double duration = note.getDurationFromRhythmString(noteRhythmValue);

                note.setRhythmValue(duration);
                if(pitch > 0)
                    note.setPitch(pitch);
                else
                    note.setPitch(Integer.MIN_VALUE);
                note.setDuration(duration);
                break;
            }
            case Full: {
                String noteName = (currentNode.contains("is")) ? currentNode.substring(0,3) : currentNode.substring(0,1);
                String noteOctave;
                String noteRhythmValue;
                if("P".equals(noteName)) {
                    noteOctave = "";
                    noteRhythmValue = currentNode.substring(2,currentNode.length());
                } else {
                    noteOctave = (currentNode.contains("is")) ? currentNode.substring(3,4) : currentNode.substring(1,2);
                    noteRhythmValue = (currentNode.contains("is")) ? currentNode.substring(5,currentNode.length()) : currentNode.substring(3,currentNode.length());
                    if (noteOctave.equals("_")) {
                        noteOctave = "";
                    }
                }

                OctaveNames octaveName = note.getOctaveFromStringSymbol(noteName, noteOctave);

                NoteNames noteNameEnum;
                int noteValue;
                int pitch;
                if (noteName.equals("P")) {
                    pitch = -1;
                } else {
                    if (octaveName.equals(OctaveNames.Mala)) {
                        noteName = noteName.length() > 1
                                ? noteName.substring(0,1).toUpperCase() + noteName.substring(1)
                                : noteName.toUpperCase();
                    }
                    noteNameEnum = NoteNames.valueOf(noteName);

                    boolean isPreviousOctave = isPreviousOctave(noteNameEnum);
                    int previousOctaveModifier = isPreviousOctave ? 0 : 1;
                    int octave = Math.max(0, octaveName.ordinal() - previousOctaveModifier);
                    noteValue = noteNameEnum.ordinal();
                    pitch = noteValue + 12 * octave;
                }

                double duration = note.getDurationFromRhythmString(noteRhythmValue);

                note.setRhythmValue(duration);
                if(pitch > 0)
                    note.setPitch(pitch);
                else
                    note.setPitch(Integer.MIN_VALUE);
                note.setDuration(duration);
                break;
            }
            default: break;
        }

        if(note.getPitch() > 0)
            note.setDynamic(70);
        else
            note.setDynamic(0);
        note.setPan(0);
        return note;
    }

    protected boolean isPreviousOctave(NoteNames noteNameEnum) {
        return noteNameEnum.equals(NoteNames.Gis)
                || noteNameEnum.equals(NoteNames.A)
                || noteNameEnum.equals(NoteNames.B)
                || noteNameEnum.equals(NoteNames.H);
    }

}
