package pl.edu.wat.wcy.isi.szymonm.phd.graph.walk;

import edu.uci.ics.jung.graph.Graph;
import pl.edu.wat.wcy.isi.szymonm.phd.enums.graphs.NoteRepresentation;
import pl.edu.wat.wcy.isi.szymonm.phd.graph.structure.WeightedEdge;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

public class EdgeDegreesWeightWalkWithSelectedStartNode extends BaseGraphWalkMethod<String, WeightedEdge> {

    private String startNode;

    public EdgeDegreesWeightWalkWithSelectedStartNode(NoteRepresentation nodeType, String startNode) {
        super(nodeType);
        this.startNode = startNode;
    }

    @Override
    protected String pickNextNode(Graph<String, WeightedEdge> graph, String currentNode) {
        String vertex = drawNextNodeBasingOnVertexDegree(graph, currentNode);
        System.out.println("Wylosowano: " + vertex);
        return vertex;
    }

    @Override
    protected String pickStartNode(Graph<String, WeightedEdge> graph) {
        return this.startNode;
    }

    private String drawNextNodeBasingOnVertexDegree(Graph<String, WeightedEdge> graph, String current) {
        int sum = getSumOfVertexSuccessorsDegrees(graph, current);
        return pickNodeFromSuccessors(graph, current, sum);
    }

    private int getSumOfVertexSuccessorsDegrees(Graph<String, WeightedEdge> graph, String current){
        int successorDegreeSum = 0;
        for(String successor: graph.getSuccessors(current)){
            WeightedEdge edge = graph.findEdge(current, successor);
            if(edge != null)
                successorDegreeSum += edge.getWeight();
        }
        return successorDegreeSum;
    }

    private String pickNodeFromSuccessors(Graph<String, WeightedEdge> graph, String current, int sumOfWeights){
        List<Double> weights = new ArrayList<>();
        List<Double> accumulatedWeights = new ArrayList<>();
        List<String> successors = new ArrayList<>(graph.getSuccessors(current));
        double accumulatedValue = 0;
        for (String successor: successors){
            WeightedEdge edge = graph.findEdge(current, successor);
            double weight = (double)edge.getWeight() / sumOfWeights;
            accumulatedValue += weight;
            weights.add(weight);
            accumulatedWeights.add(accumulatedValue);
        }
        double drawnValue = new Random().nextDouble();

        for(int i = 0; i < accumulatedWeights.size(); i++){
            if(accumulatedWeights.get(i) > drawnValue)
                return successors.get(i);
        }

        throw new IllegalArgumentException("Przekroczono wartość 1 podczas losowania!");
    }
}
