package pl.edu.wat.wcy.isi.szymonm.phd.graph.walk;

import edu.uci.ics.jung.graph.Graph;
import pl.edu.wat.wcy.isi.szymonm.phd.enums.graphs.NoteRepresentation;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

public class VertexDegreesWeightWalkWithMaxDegreeStartNode extends BaseGraphWalkMethod<String, String> {

    public VertexDegreesWeightWalkWithMaxDegreeStartNode(NoteRepresentation nodeType) {
        super(nodeType);
    }

    @Override
    protected String pickNextNode(Graph<String, String> graph, String currentNode) {
        String vertex = drawNextNodeBasingOnVertexDegree(graph, currentNode);
        System.out.println("Wylosowano: " + vertex);
        return vertex;
    }

    @Override
    protected String pickStartNode(Graph<String, String> graph) {
        String vertex = pickNodeWithMaxDegree(graph);
        System.out.println("Start: " + vertex);
        return vertex;
    }

    private String drawNextNodeBasingOnVertexDegree(Graph<String, String> graph, String current) {
        if(current != null){ //pick one of current's successors
            int sum = getSumOfVertexSuccessorsDegrees(graph, current);
            return pickNodeFromSuccessors(graph, current, sum);
        } else { //pick one from all graph
            int sum = getSumOfGraphVerticesDegrees(graph);
            return pickNodeFromGraph(graph, sum);
        }
    }

    private int getSumOfVertexSuccessorsDegrees(Graph<String, String> graph, String current){
        int successorDegreeSum = 0;
        for(String successor: graph.getSuccessors(current)){
            successorDegreeSum += graph.getSuccessorCount(successor);
        }
        return successorDegreeSum;
    }

    private int getSumOfGraphVerticesDegrees(Graph<String, String> graph){
        int vertexDegreeSum = 0;
        for(String vertex: graph.getVertices()){
            vertexDegreeSum += graph.getSuccessorCount(vertex);
        }
        return vertexDegreeSum;
    }

    private String pickNodeFromSuccessors(Graph<String, String> graph, String current, int sumOfWeights){
        List<Double> weights = new ArrayList<>();
        List<Double> accumulatedWeights = new ArrayList<>();
        List<String> successors = new ArrayList<>(graph.getSuccessors(current));
        double accumulatedValue = 0;
        for (String successor: successors){
            double weight = (double)graph.degree(successor) / sumOfWeights;
            accumulatedValue += weight;
            weights.add(weight);
            accumulatedWeights.add(accumulatedValue);
        }
        double drawnValue = new Random().nextDouble();

        for(int i = 0; i < accumulatedWeights.size(); i++){
            if(accumulatedWeights.get(i) > drawnValue)
                return successors.get(i);
        }

        throw new IllegalArgumentException("Przekroczono wartość 1 podczas losowania!");
    }

    private String pickNodeFromGraph(Graph<String, String> graph, int sumOfWeights){
        List<Double> weights = new ArrayList<>();
        List<Double> accumulatedWeights = new ArrayList<>();
        List<String> vertices = new ArrayList<>(graph.getVertices());
        double accumulatedValue = 0;
        for (String vertex: vertices){
            double weight = (double)graph.degree(vertex) / sumOfWeights;
            accumulatedValue += weight;
            weights.add(weight);
            accumulatedWeights.add(accumulatedValue);
        }
        double drawnValue = new Random().nextDouble();

        for(int i = 0; i < accumulatedWeights.size(); i++){
            if(accumulatedWeights.get(i) > drawnValue)
                return vertices.get(i);
        }

        throw new IllegalArgumentException("Przekroczono wartość 1 podczas losowania!");
    }

    private String pickNodeWithMaxDegree(Graph<String, String> graph){
        List<String> vertices = new ArrayList<>(graph.getVertices());

        int maxValue = 0;
        String maxVertex = null;

        for (String vertex: vertices){
            int degree = graph.degree(vertex);
            if(degree > maxValue)
                maxVertex = vertex;
        }

        return maxVertex;
    }
}
