package pl.edu.wat.wcy.isi.szymonm.phd.musicxml;

import edu.uci.ics.jung.graph.UndirectedGraph;
import org.junit.Before;
import org.junit.Test;
import pl.edu.wat.wcy.isi.szymonm.phd.graph.adjacency.GraphAdjacencyMatrixAssembler;
import pl.edu.wat.wcy.isi.szymonm.phd.graph.structure.UndirectedGraphSimpleNotesCreation;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.nio.file.Files;
import java.util.Collections;
import java.util.List;

import static org.junit.Assert.assertEquals;

public class AdjacencyMatrixCreationTest {

    private static final int ALL_POSSIBLE_OCTAVE_NOTES_COUNT = 13;

    @Before
    public void init(){

    }

    @Test
    public void shouldParseEmptyFile() throws Exception {
        //given
        Score score = getScoreFromTestResources("EmptyScore.xml");
        Phrase phrase = score.getPartList().get(0).getPhraseList().get(0);
        List<Note> noteList = phrase.getNoteList() == null ? Collections.emptyList() : phrase.getNoteList() ;

        UndirectedGraphSimpleNotesCreation graphSimpleNotesCreation = new UndirectedGraphSimpleNotesCreation();
        UndirectedGraph<String, String> undirectedGraph = graphSimpleNotesCreation.addNotesToGraph(noteList);

        //when
        GraphAdjacencyMatrixAssembler graphAdjacencyMatrixAssembler = new GraphAdjacencyMatrixAssembler();
        graphAdjacencyMatrixAssembler.createAdjacencyMatrix(undirectedGraph);

        //then
        assertEquals(0, noteList.size());
        //manually check the output
    }

    @Test
    public void shouldParseMelodyFile() throws Exception {
        //given
        Score score = getScoreFromTestResources("comparisons_sample/RollingInTheDeep_sample.xml");
        Phrase phrase = score.getPartList().get(0).getPhraseList().get(0);
        List<Note> noteList = phrase.getNoteList() == null ? Collections.emptyList() : phrase.getNoteList() ;

        UndirectedGraphSimpleNotesCreation graphSimpleNotesCreation = new UndirectedGraphSimpleNotesCreation();
        UndirectedGraph<String, String> undirectedGraph = graphSimpleNotesCreation.addNotesToGraph(noteList, true);

        //when
        GraphAdjacencyMatrixAssembler graphAdjacencyMatrixAssembler = new GraphAdjacencyMatrixAssembler();
        graphAdjacencyMatrixAssembler.createAdjMatrix(undirectedGraph);

        //then
        assertEquals(26, noteList.size());
        assertEquals(ALL_POSSIBLE_OCTAVE_NOTES_COUNT, undirectedGraph.getVertexCount());
        //manually check the output
    }

    private Score getScoreFromTestResources(String fileName) throws IOException, JAXBException {
        URL fileUrl = this.getClass().getClassLoader().getResource(fileName);
        URI fileUri = null;
        try {
            fileUri =  fileUrl != null ? fileUrl.toURI() : null;
        } catch (URISyntaxException e) { }
        String filename = fileUri != null ? fileUri.getPath() : null;
        String content = new String(Files.readAllBytes(new File(filename).toPath()));

        JAXBContext context = JAXBContext.newInstance(Score.class);
        Unmarshaller um = context.createUnmarshaller();
        Score score = (Score) um.unmarshal(new File(filename));
        return score;
    }

}
