package pl.edu.wat.wcy.isi.szymonm.phd.musicxml;

import edu.uci.ics.jung.graph.DirectedGraph;
import org.junit.Ignore;
import org.junit.Test;
import pl.edu.wat.wcy.isi.szymonm.phd.graph.adjacency.GraphAdjacencyMatrixAssembler;
import pl.edu.wat.wcy.isi.szymonm.phd.graph.structure.DirectedGraphSimpleNotesCreation;
import pl.edu.wat.wcy.isi.szymonm.phd.graph.structure.DirectedNetworkShortNotesCreation;
import pl.edu.wat.wcy.isi.szymonm.phd.graph.structure.WeightedEdge;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.nio.file.*;
import java.time.LocalDate;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.util.Collections;
import java.util.List;

import static org.junit.Assert.assertEquals;

public class MatrixToExcelCopier {

    public static final String FILE_BASE_NAME = "sieci_sample";
    public static final String FILE_EXTENSION = ".xlsm";
    private static final int ALL_POSSIBLE_OCTAVE_NOTES_COUNT = 13;

    @Test
    @Ignore
    public void shouldParseMelodyFileAndSaveInExcelAndOpenSheet() throws Exception {
        Path basePath = Paths.get(getFileUri(FILE_BASE_NAME + FILE_EXTENSION));
        URI copyUri = URI.create(basePath.toUri().toString().replace(FILE_BASE_NAME,
                FILE_BASE_NAME + "_" + LocalDate.now().format(DateTimeFormatter.ISO_LOCAL_DATE)
                        + "_" + LocalTime.now().format(DateTimeFormatter.ofPattern("HHmmss"))));
        Path copyPath = Paths.get(copyUri);
        Path file = Files.copy(basePath, copyPath, StandardCopyOption.COPY_ATTRIBUTES, StandardCopyOption.REPLACE_EXISTING);
        File excelFile = new File(file.toUri());
        System.out.println(excelFile.exists());

        runGraphTestForFile("TestScoreWithMelody.xml", excelFile,0, 9);
        runGraphTestForFile("TestScoreWithMelody2.xml", excelFile, 1, 6);
    }

    @Test
    @Ignore
    public void shouldParseWlazlKotekAndSaveInExcelAndOpenSheet() throws Exception {
        Path basePath = Paths.get(getFileUri(FILE_BASE_NAME + FILE_EXTENSION));
        URI copyUri = URI.create(basePath.toUri().toString().replace(FILE_BASE_NAME,
                FILE_BASE_NAME + "_" + LocalDate.now().format(DateTimeFormatter.ISO_LOCAL_DATE)
                        + "_" + LocalTime.now().format(DateTimeFormatter.ofPattern("HHmmss"))));
        Path copyPath = Paths.get(copyUri);
        Path file = Files.copy(basePath, copyPath, StandardCopyOption.COPY_ATTRIBUTES, StandardCopyOption.REPLACE_EXISTING);
        File excelFile = new File(file.toUri());
        System.out.println(excelFile.exists());

        runGraphTestForFile("WlazłKotek_CDur.xml", excelFile,0, 24);
        runGraphTestForFile("WlazłKotek_DDur.xml", excelFile, 1, 24);
    }

    // TODO szymonm osobny test: toString wygenerowany przez generowanie macierzy z zadanym wzorcem dla wybranego utworu

    private void runGraphTestForFile(String melodyFileName, File excelFile, int partNumber, int expNoteCount) throws IOException, JAXBException {
        //given
        Score score = getScoreFromTestResources(melodyFileName);
        Phrase phrase = score.getPartList().get(0).getPhraseList().get(0);
        List<Note> noteList = phrase.getNoteList() == null ? Collections.emptyList() : phrase.getNoteList() ;
        refreshFrequencyForAllNotes(noteList);
        DirectedGraphSimpleNotesCreation graphSimpleNotesCreation = new DirectedGraphSimpleNotesCreation();
        DirectedGraph<String, String> directedGraph = graphSimpleNotesCreation.addNotesToGraph(noteList, true);

        //when
        GraphAdjacencyMatrixAssembler graphAdjacencyMatrixAssembler = new GraphAdjacencyMatrixAssembler();
        graphAdjacencyMatrixAssembler.createAdjMatrixWithExcelFile(directedGraph, excelFile, partNumber);

        //then
        assertEquals(expNoteCount, noteList.size());
        assertEquals(ALL_POSSIBLE_OCTAVE_NOTES_COUNT, directedGraph.getVertexCount());
    }

    private void refreshFrequencyForAllNotes(List<Note> noteList) {
        // adjusts frequency using pitch
        noteList.forEach(note -> note.setPitch(note.getPitch()));
    }


    @Test
    @Ignore
    public void shouldParseMelodyFileAndSaveNetworkInExcelAndOpenSheet() throws Exception {
        Path basePath = Paths.get(getFileUri(FILE_BASE_NAME + FILE_EXTENSION));
        URI copyUri = URI.create(basePath.toUri().toString().replace(FILE_BASE_NAME,
                FILE_BASE_NAME + "_" + LocalDate.now().format(DateTimeFormatter.ISO_LOCAL_DATE)
                        + "_" + LocalTime.now().format(DateTimeFormatter.ofPattern("HHmmss"))));
        Path copyPath = Paths.get(copyUri);
        Path file = Files.copy(basePath, copyPath, StandardCopyOption.COPY_ATTRIBUTES, StandardCopyOption.REPLACE_EXISTING);
        File excelFile = new File(file.toUri());
        System.out.println(excelFile.exists());

        runNetworkTestForFile("TestScoreWithMelody.xml", excelFile,0, 9);
        runNetworkTestForFile("TestScoreWithMelody3.xml", excelFile, 1, 14);
    }

    private void runNetworkTestForFile(String melodyFileName, File excelFile, int partNumber, int expNoteCount) throws IOException, JAXBException {
        //given
        Score score = getScoreFromTestResources(melodyFileName);
        Phrase phrase = score.getPartList().get(0).getPhraseList().get(0);
        List<Note> noteList = phrase.getNoteList() == null ? Collections.emptyList() : phrase.getNoteList() ;
        refreshFrequencyForAllNotes(noteList);

        DirectedNetworkShortNotesCreation networkSimpleNotesCreation = new DirectedNetworkShortNotesCreation();
        DirectedGraph<String, WeightedEdge> directedGraph = networkSimpleNotesCreation.addNotesToNetwork(noteList, true);

        //when
        GraphAdjacencyMatrixAssembler graphAdjacencyMatrixAssembler = new GraphAdjacencyMatrixAssembler();
        graphAdjacencyMatrixAssembler.createAdjMatrixForNetworkWithExcelFile(directedGraph, excelFile, partNumber);

        //then
        assertEquals(expNoteCount, noteList.size());
        assertEquals(ALL_POSSIBLE_OCTAVE_NOTES_COUNT, directedGraph.getVertexCount());
    }

    private Score getScoreFromTestResources(String fileName) throws IOException, JAXBException {
        URI fileUri = getFileUri(fileName);
        String filename = fileUri != null ? fileUri.getPath() : null;
        String content = new String(Files.readAllBytes(new File(filename).toPath()));

        JAXBContext context = JAXBContext.newInstance(Score.class);
        Unmarshaller um = context.createUnmarshaller();
        Score score = (Score) um.unmarshal(new File(filename));
        return score;
    }

    private URI getFileUri(String fileName) {
        URL fileUrl = this.getClass().getClassLoader().getResource(fileName);
        URI fileUri = null;
        try {
            fileUri =  fileUrl != null ? fileUrl.toURI() : null;
        } catch (URISyntaxException e) { }
        return fileUri;
    }

}
