package pl.edu.wat.wcy.isi.szymonm.phd.musicxml;

import edu.uci.ics.jung.graph.DirectedGraph;
import org.apache.commons.compress.utils.Charsets;
import org.apache.commons.io.IOUtils;
import org.junit.Ignore;
import org.junit.Test;
import pl.edu.wat.wcy.isi.szymonm.phd.graph.adjacency.GraphAdjacencyMatrixAssembler;
import pl.edu.wat.wcy.isi.szymonm.phd.graph.structure.*;
import pl.edu.wat.wcy.isi.szymonm.phd.visualisation.arrangement.EdgeDegreeRandomWalkMultipleArrangementGenerator;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardCopyOption;
import java.util.Collections;
import java.util.List;

import static org.junit.Assert.assertEquals;

public class PiecesComparingTest extends MatrixToExcelCopier {

    public static final String FILE_BASE_NAME = "sieci_sample";
    public static final String FILE_EXTENSION = ".xlsm";
    public static final DatasetType DATASET = DatasetType.SAMPLE;
    private static String CURRENT_SUBFOLDER = "";
    private static final String FOLDER_TO_COMPARE = "./comparisons_sample";
    private static final int ARRANGEMENTS_COUNT = 3;
    private enum DatasetType { SAMPLE, FULL }

    @Test
    @Ignore
    public void shouldGetFilesToCompareAndOpenSheets() throws Exception {
        ClassLoader classLoader = this.getClass().getClassLoader();
        String folderName = FOLDER_TO_COMPARE;


        InputStream folderStream = classLoader.getResourceAsStream(folderName);
        List<String> fileNames = IOUtils.readLines(folderStream, Charsets.UTF_8.toString());

        switch(DATASET){
            case SAMPLE:
                compareAll(fileNames);
                break;
            case FULL:
            default:
                compareSubfolders(folderName, fileNames);
                // TODO szymonm compare a single delegate from each folder with other folders
                break;
        }

        //Files.list(Paths.get(classLoader.getResource(FOLDER_TO_COMPARE).toURI())).collect(Collectors.toList());
        //Collection<File> files = FileUtils.listFiles(new File(FOLDER_TO_COMPARE + "/"), null, false);
        //copySheetAndOpenFiles("TestScoreWithMelody.xml", "TestScoreWithMelody2.xml");
    }

    private void compareAll(List<String> fileNames) throws Exception {
        for (String fileName : fileNames) {
            for (String fileNameToCompareWith : fileNames) {
                if (fileName.compareTo(fileNameToCompareWith) > 0) {
                    // TODO szymonm make a separate method for comparing with both options
                    //copySheetAndOpenFiles(fileName, fileNameToCompareWith);
                    copySheetAndOpenFilesForNetworks(fileName, fileNameToCompareWith);
                }
            }
        }
    }

    private void compareSubfolders(String mainFolderName, List<String> subfolderNames) throws Exception {
        ClassLoader classLoader = this.getClass().getClassLoader();
        for (String subfolderName : subfolderNames) {
            CURRENT_SUBFOLDER = "/" + subfolderName;
            InputStream folderStream = classLoader.getResourceAsStream(mainFolderName + "/" + subfolderName);
            List<String> fileNames = IOUtils.readLines(folderStream, Charsets.UTF_8.toString());
            compareAll(fileNames);
        }
    }

    @Test
    @Ignore
    public void generateArrangementsForPieces() throws Exception {
        ClassLoader classLoader = this.getClass().getClassLoader();
        String folderName = FOLDER_TO_COMPARE;


        InputStream folderStream = classLoader.getResourceAsStream(folderName);
        List<String> fileNames = IOUtils.readLines(folderStream, Charsets.UTF_8.toString());

        switch(DATASET){
            case SAMPLE:
                generateAllArrangements(fileNames);
                break;
            case FULL:
            default:
                generateArrangementsFromSubfolders(folderName, fileNames);
                // TODO szymonm compare a single delegate from each folder with other folders
                break;
        }

        //Files.list(Paths.get(classLoader.getResource(FOLDER_TO_COMPARE).toURI())).collect(Collectors.toList());
        //Collection<File> files = FileUtils.listFiles(new File(FOLDER_TO_COMPARE + "/"), null, false);
        //copySheetAndOpenFiles("TestScoreWithMelody.xml", "TestScoreWithMelody2.xml");
    }

    private void generateAllArrangements(List<String> fileNames) throws Exception {
        for (String fileName : fileNames) {
            String resourceRelativePath = FOLDER_TO_COMPARE + CURRENT_SUBFOLDER + "/" + fileName;
            Path basePath = Paths.get(getFileUri(resourceRelativePath));
            EdgeDegreeRandomWalkMultipleArrangementGenerator.generateArrangements(basePath.toString(), resourceRelativePath, ARRANGEMENTS_COUNT);
        }
        Thread.sleep(120 * 1000);
    }

    private void generateArrangementsFromSubfolders(String mainFolderName, List<String> subfolderNames) throws Exception {
        ClassLoader classLoader = this.getClass().getClassLoader();
        for (String subfolderName : subfolderNames) {
            CURRENT_SUBFOLDER = "/" + subfolderName;
            InputStream folderStream = classLoader.getResourceAsStream(mainFolderName + "/" + subfolderName);
            List<String> fileNames = IOUtils.readLines(folderStream, Charsets.UTF_8.toString());
            generateAllArrangements(fileNames);
        }
    }

    private void copySheetAndOpenFiles(String melodyFileName1, String melodyFileName2) throws IOException, JAXBException {
        File excelFile = openAndCopyExcelFile(melodyFileName1, melodyFileName2);

        runGraphTestForFile(melodyFileName1, excelFile,0, 0, true);
        runGraphTestForFile(melodyFileName2, excelFile, 1, 0, true);
    }

    private void copySheetAndOpenFilesForNetworks(String melodyFileName1, String melodyFileName2) throws IOException, JAXBException {
        File excelFile = openAndCopyExcelFile(melodyFileName1, melodyFileName2);

        runNetworkTestForFile(melodyFileName1, excelFile,0, 0, true);
        runNetworkTestForFile(melodyFileName2, excelFile, 1, 0, true);
    }

    private File openAndCopyExcelFile(String melodyFileName1, String melodyFileName2) throws IOException {
        Path basePath = Paths.get(getFileUri(FILE_BASE_NAME + FILE_EXTENSION));
        String melodyFileNameReplaced1 = melodyFileName1.replace(" ", "_");
        String melodyFileNameReplaced2 = melodyFileName2.replace(" ", "_");
        URI copyUri = URI.create(basePath.toUri().toString().replace(FILE_BASE_NAME,
                FILE_BASE_NAME + "_comparison_" + melodyFileNameReplaced1 + "_and_" + melodyFileNameReplaced2));
        Path copyPath = Paths.get(copyUri);
        Path file = Files.copy(basePath, copyPath, StandardCopyOption.COPY_ATTRIBUTES, StandardCopyOption.REPLACE_EXISTING);
        File excelFile = new File(file.toUri());
        System.out.println(excelFile.exists());
        return excelFile;
    }

    // TODO szymonm osobny test: toString wygenerowany przez generowanie macierzy z zadanym wzorcem dla wybranego utworu

    private void runGraphTestForFile(String melodyFileName, File excelFile, int partNumber, int expNoteCount)
            throws IOException, JAXBException {
        runGraphTestForFile(melodyFileName, excelFile, partNumber, expNoteCount, false);
    }

    private void runGraphTestForFile(String melodyFileName, File excelFile, int partNumber, int expNoteCount,
                                     boolean ignoreNoteCountCheck) throws IOException, JAXBException {
        //given
        Score score = getScoreFromTestResources(melodyFileName);
        Phrase phrase = score.getPartList().get(0).getPhraseList().get(0);
        List<Note> noteList = phrase.getNoteList() == null ? Collections.emptyList() : phrase.getNoteList() ;
        refreshFrequencyForAllNotes(noteList);
        DirectedGraphSimpleNotesCreation graphSimpleNotesCreation = new DirectedGraphSimpleNotesCreation();
        DirectedGraph<String, String> directedGraph = graphSimpleNotesCreation.addNotesToGraph(noteList, true);

        //when
        GraphAdjacencyMatrixAssembler graphAdjacencyMatrixAssembler = new GraphAdjacencyMatrixAssembler();
        graphAdjacencyMatrixAssembler.createAdjMatrixWithExcelFile(directedGraph, excelFile, partNumber);

        //then
        if (!ignoreNoteCountCheck) {
            assertEquals(expNoteCount, noteList.size());
        }
        assertEquals(13, directedGraph.getVertexCount());
    }

    private void refreshFrequencyForAllNotes(List<Note> noteList) {
        // adjusts frequency using pitch
        noteList.forEach(note -> note.setPitch(note.getPitch()));
    }

    private void runNetworkTestForFile(String melodyFileName, File excelFile, int partNumber, int expNoteCount,
                                       boolean ignoreNoteCountCheck) throws IOException, JAXBException {
        //given
        Score score = getScoreFromTestResources(melodyFileName);
        Phrase phrase = score.getPartList().get(0).getPhraseList().get(0);
        List<Note> noteList = phrase.getNoteList() == null ? Collections.emptyList() : phrase.getNoteList() ;
        refreshFrequencyForAllNotes(noteList);

        DirectedNetworkWithWeightedVerticesShortNotesCreation networkSimpleNotesCreation = new DirectedNetworkWithWeightedVerticesShortNotesCreation();
        DirectedGraph<WeightedVertex, WeightedEdge> directedGraph = networkSimpleNotesCreation.addNotesToWeightedNetwork(noteList);

        //when
        GraphAdjacencyMatrixAssembler graphAdjacencyMatrixAssembler = new GraphAdjacencyMatrixAssembler();
        graphAdjacencyMatrixAssembler.createAdjMatrixForWeightedNetworkWithExcelFile(directedGraph, excelFile, partNumber);

        //then
        if (!ignoreNoteCountCheck) {
            assertEquals(expNoteCount, noteList.size());
        }
        assertEquals(13, directedGraph.getVertexCount());
    }

    private Score getScoreFromTestResources(String fileName) throws IOException, JAXBException {
        URI fileUri = getFileUri(FOLDER_TO_COMPARE + CURRENT_SUBFOLDER + "/" + fileName);
        String filename = fileUri != null ? fileUri.getPath() : null;
        String content = new String(Files.readAllBytes(new File(filename).toPath()));

        JAXBContext context = JAXBContext.newInstance(Score.class);
        Unmarshaller um = context.createUnmarshaller();
        Score score = (Score) um.unmarshal(new File(filename));
        return score;
    }

    private URI getFileUri(String fileName) {
        URL fileUrl = this.getClass().getClassLoader().getResource(fileName);
        URI fileUri = null;
        try {
            fileUri =  fileUrl != null ? fileUrl.toURI() : null;
        } catch (URISyntaxException e) { }
        return fileUri;
    }

}
